import { newSpecPage } from "@stencil/core/testing";
import { MxAutocomplete } from "../mx-autocomplete";
import { MxBlockInput } from "../../mx-block-input/mx-block-input";
import { MxMenu } from "../../mx-menu/mx-menu";
import { MxMenuItem } from "../../mx-menu-item/mx-menu-item";
describe('mx-autocomplete', () => {
  let page;
  let root;
  let input;
  beforeEach(async () => {
    page = await newSpecPage({
      components: [MxAutocomplete, MxBlockInput, MxMenu, MxMenuItem],
      html: `
        <mx-autocomplete
          label="Search"
          placeholder="Search..."
          option-label="name"
          option-icon="mds-check"
          filter-options
        />
      `,
    });
    root = page.root;
    root.options = ['Option 1', 'Option 2', 'Option 3'];
    input = root.querySelector('input');
  });
  it('updates the input value on user input', async () => {
    input.value = 'Option';
    input.dispatchEvent(new Event('input', { bubbles: true }));
    await page.waitForChanges();
    expect(root.value).toBe('Option');
  });
  it('filters options based on input', async () => {
    input.value = '1';
    input.dispatchEvent(new Event('input', { bubbles: true }));
    await page.waitForChanges();
    expect(root.querySelectorAll('mx-menu-item').length).toBe(1);
    expect(root.querySelector('mx-menu-item').textContent).toContain('Option 1');
  });
  it('filters and renders options by the option-label property', async () => {
    root.options = [{ name: 'Option 1' }, { name: 'Option 2' }, { name: 'Option 3' }];
    root.optionLabel = 'name';
    await page.waitForChanges();
    input.value = '1';
    input.dispatchEvent(new Event('input', { bubbles: true }));
    await page.waitForChanges();
    expect(root.querySelectorAll('mx-menu-item').length).toBe(1);
    expect(root.querySelector('mx-menu-item').textContent.trim()).toBe('Option 1');
  });
  it('filters using a custom filterOptions function', async () => {
    root.options = [{ name: 'Option 1' }, { name: 'Option 2' }, { name: 'Option 3' }];
    root.filterOptions = (v, o) => !o.name.includes(v);
    root.optionLabel = 'name';
    await page.waitForChanges();
    input.value = '1';
    input.dispatchEvent(new Event('input', { bubbles: true }));
    await page.waitForChanges();
    expect(root.querySelectorAll('mx-menu-item').length).toBe(2);
    expect(root.querySelector('mx-menu-item').textContent.trim()).toBe('Option 2');
  });
  it('assigns the icon based on the optionIcon prop', async () => {
    root.optionIcon = 'icon-class';
    await page.waitForChanges();
    input.value = '1';
    input.dispatchEvent(new Event('input', { bubbles: true }));
    await page.waitForChanges();
    const menuItem = root.querySelector('mx-menu-item');
    expect(menuItem.icon).toBe('icon-class');
  });
  it('assigns icon using a custom optionIcon function', async () => {
    root.optionIcon = o => 'icon-' + o.slice(-1);
    await page.waitForChanges();
    input.value = '1';
    input.dispatchEvent(new Event('input', { bubbles: true }));
    await page.waitForChanges();
    expect(root.querySelector('mx-menu-item').icon).toBe('icon-1');
  });
  it('renders option groups', async () => {
    root.options = [
      ['Option 1', 'Option 2'],
      ['Option 3', 'Option 4'],
    ];
    root.optionGroups = ['Group 1', 'Group 2'];
    await page.waitForChanges();
    input.value = 'Option';
    input.dispatchEvent(new Event('input', { bubbles: true }));
    await page.waitForChanges();
    expect(root.querySelectorAll('[role="group"]').length).toBe(2);
    expect(root.querySelectorAll('[role="group"] mx-menu-item').length).toBe(4);
    expect(root.querySelectorAll('[role="heading"]').length).toBe(2);
    expect(root.querySelector('[role="heading"]').textContent.trim()).toBe('Group 1');
  });
  it('renders a custom inputTag', async () => {
    root.inputTag = 'mx-input';
    await page.waitForChanges();
    expect(root.querySelector('mx-input')).not.toBeNull();
    expect(root.querySelector('mx-block-input')).toBeNull();
  });
  it('emits an mxSelect event when an option is selected', async () => {
    const mxSelectListener = jest.fn();
    root.addEventListener('mxSelect', mxSelectListener);
    input.value = 'Option';
    input.dispatchEvent(new Event('input', { bubbles: true }));
    await page.waitForChanges();
    root.querySelector('mx-menu-item').click();
    expect(mxSelectListener).toHaveBeenCalled();
  });
  it('shows no results message when no options match', async () => {
    input.value = 'blah';
    input.dispatchEvent(new Event('input', { bubbles: true }));
    await page.waitForChanges();
    expect(root.querySelectorAll('mx-menu-item').length).toBe(1);
    expect(root.querySelector('[data-test="no-results"]')).not.toBeNull();
  });
});
//# sourceMappingURL=mx-autocomplete.spec.js.map
