import { Host, h } from "@stencil/core";
import { uuidv4 } from "../../utils/utils";
export class MxBlockFileInput {
  constructor() {
    this.uuid = uuidv4();
    this.accept = undefined;
    this.acceptImage = false;
    this.acceptPdf = false;
    this.assistiveText = undefined;
    this.chooseBtnType = 'contained';
    this.chooseFileText = 'Choose File';
    this.colspan = undefined;
    this.disabled = false;
    this.error = false;
    this.filename = undefined;
    this.inputId = undefined;
    this.label = undefined;
    this.name = undefined;
    this.placeholder = 'No file chosen';
    this.readonly = false;
    this.thumbnailSize = 'cover';
    this.thumbnailUrl = undefined;
    this.thumbnailDataUri = undefined;
  }
  onThumbnailUrlChange() {
    this.mxThumbnailChange.emit(this.thumbnailUrl);
  }
  connectedCallback() {
    this.onThumbnailUrlChange();
  }
  async removeFile() {
    if (this.disabled || this.readonly)
      return;
    if (!this.isFileSelected)
      return;
    this.filename = null;
    this.thumbnailUrl = null;
    this.fileInput.value = '';
    this.fileInput.dispatchEvent(new window.Event('change', { bubbles: true, cancelable: true }));
    this.fileInput.dispatchEvent(new window.Event('input', { bubbles: true, cancelable: true }));
  }
  async selectFile() {
    if (this.disabled || this.readonly)
      return;
    this.fileInput.click();
  }
  onFocus() {
    this.error = false;
  }
  onInput(e) {
    this.error = false;
    if (this.fileInput.value) {
      this.filename = e.target.files[0].name;
      this.setThumnailDataUri(e.target.files[0]);
    }
    else {
      this.filename = null;
      this.thumbnailUrl = null;
      this.thumbnailDataUri = null;
    }
  }
  setThumnailDataUri(file) {
    this.thumbnailDataUri = null;
    if (!/\.(jpe?g|png|gif)$/i.test(file.name))
      return;
    const reader = new FileReader();
    reader.onload = () => {
      this.thumbnailDataUri = reader.result;
      this.mxThumbnailChange.emit(this.thumbnailDataUri);
    };
    reader.readAsDataURL(file);
  }
  getElementId(suffix = '') {
    if (suffix)
      suffix = '-' + suffix;
    return (this.inputId || this.uuid) + suffix;
  }
  get isFileSelected() {
    if (this.filename || this.thumbnailUrl)
      return true;
    if (this.fileInput && this.fileInput.value)
      return true;
    return false;
  }
  get acceptString() {
    const accepts = [];
    if (this.acceptImage)
      accepts.push('image/*');
    if (this.acceptPdf)
      accepts.push('.pdf');
    if (this.accept)
      accepts.push(this.accept);
    return accepts.join(',') || null;
  }
  get thumbnailBackgroundImage() {
    let url = this.thumbnailUrl;
    if (this.fileInput && this.fileInput.value)
      url = this.thumbnailDataUri;
    if (!url)
      return null;
    return `url(${url})`;
  }
  get thumbnailBackgroundSize() {
    if (!['contain', 'cover', 'auto', 'stretch'].includes(this.thumbnailSize))
      return 'cover';
    if (this.thumbnailSize === 'stretch')
      return '100% 100%';
    return this.thumbnailSize;
  }
  get hostClass() {
    let str = 'mx-block-input mx-block-file-input';
    if (this.colspan > 0 && this.colspan < 7) {
      str += ' col-span-' + this.colspan;
    }
    return str;
  }
  get containerClass() {
    let str = 'mx-block-input-container';
    if (this.disabled || this.readonly)
      str += ' disabled';
    if (this.error)
      str += ' error';
    return str;
  }
  render() {
    const removeButton = !this.disabled && !this.readonly && this.isFileSelected && (h("button", { type: "button", class: "clear-button flex items-center justify-center w-20 text-1", onClick: this.removeFile.bind(this), "aria-label": "Remove file", title: "Remove file", "data-testid": "remove-file-button" }, h("i", { class: "mds-x" })));
    return (h(Host, { class: this.hostClass }, h("div", { class: this.containerClass }, h("input", { ref: el => (this.fileInput = el), class: "absolute pointer-events-none opacity-0", id: this.getElementId(), name: this.name, "aria-describedby": [this.getElementId('assistive'), this.getElementId('filename')].join(' '), type: "file", accept: this.acceptString, disabled: this.disabled, readonly: this.readonly, tabindex: "-1", onInput: this.onInput.bind(this) }), this.isFileSelected && this.thumbnailBackgroundImage && (h("div", { "data-testid": "thumbnail", class: "thumbnail w-80 h-80 bg-center bg-no-repeat flex-shrink-0", style: { backgroundImage: this.thumbnailBackgroundImage, backgroundSize: this.thumbnailBackgroundSize } })), h("div", { class: "mx-block-input-grid" }, h("label", { class: "text-small", htmlFor: this.getElementId(), onClick: e => e.preventDefault() }, this.label, h("slot", { name: "label" }), this.assistiveText && h("span", { id: this.getElementId('assistive') }, this.assistiveText)), h("span", { id: this.getElementId('filename'), class: "input-value truncate", "aria-live": "polite", title: this.filename, "data-testid": "filename" }, !this.isFileSelected && h("span", { class: "opacity-60" }, this.placeholder), this.filename), removeButton, h("mx-button", { "btn-type": this.chooseBtnType, hug: true, small: true, disabled: this.disabled || this.readonly, onClick: this.selectFile.bind(this), "data-testid": "choose-file-button" }, this.chooseFileText)))));
  }
  static get is() { return "mx-block-file-input"; }
  static get properties() {
    return {
      "accept": {
        "type": "string",
        "mutable": true,
        "complexType": {
          "original": "string",
          "resolved": "string",
          "references": {}
        },
        "required": false,
        "optional": false,
        "docs": {
          "tags": [],
          "text": "The `accept` attribute for the file input"
        },
        "attribute": "accept",
        "reflect": true
      },
      "acceptImage": {
        "type": "boolean",
        "mutable": false,
        "complexType": {
          "original": "boolean",
          "resolved": "boolean",
          "references": {}
        },
        "required": false,
        "optional": false,
        "docs": {
          "tags": [],
          "text": "For convenience, adds \"image/*\" to the `accept` string"
        },
        "attribute": "accept-image",
        "reflect": false,
        "defaultValue": "false"
      },
      "acceptPdf": {
        "type": "boolean",
        "mutable": false,
        "complexType": {
          "original": "boolean",
          "resolved": "boolean",
          "references": {}
        },
        "required": false,
        "optional": false,
        "docs": {
          "tags": [],
          "text": "For convenience, adds \".pdf\" to the `accept` string"
        },
        "attribute": "accept-pdf",
        "reflect": false,
        "defaultValue": "false"
      },
      "assistiveText": {
        "type": "string",
        "mutable": false,
        "complexType": {
          "original": "string",
          "resolved": "string",
          "references": {}
        },
        "required": false,
        "optional": false,
        "docs": {
          "tags": [],
          "text": "Assistive text to display next to the label"
        },
        "attribute": "assistive-text",
        "reflect": false
      },
      "chooseBtnType": {
        "type": "string",
        "mutable": false,
        "complexType": {
          "original": "BtnType",
          "resolved": "\"action\" | \"contained\" | \"outlined\" | \"simple\" | \"text\"",
          "references": {
            "BtnType": {
              "location": "import",
              "path": "../mx-button/mx-button",
              "id": "src/components/mx-button/mx-button.tsx::BtnType"
            }
          }
        },
        "required": false,
        "optional": false,
        "docs": {
          "tags": [],
          "text": "The [`btnType` prop](/components/buttons.html) for the Choose File button."
        },
        "attribute": "choose-btn-type",
        "reflect": false,
        "defaultValue": "'contained'"
      },
      "chooseFileText": {
        "type": "string",
        "mutable": false,
        "complexType": {
          "original": "string",
          "resolved": "string",
          "references": {}
        },
        "required": false,
        "optional": false,
        "docs": {
          "tags": [],
          "text": "The inner text for Choose File button"
        },
        "attribute": "choose-file-text",
        "reflect": false,
        "defaultValue": "'Choose File'"
      },
      "colspan": {
        "type": "number",
        "mutable": false,
        "complexType": {
          "original": "number",
          "resolved": "number",
          "references": {}
        },
        "required": false,
        "optional": false,
        "docs": {
          "tags": [],
          "text": "Used to define the number of columns to span in the parent wrapper if more than 1"
        },
        "attribute": "colspan",
        "reflect": false
      },
      "disabled": {
        "type": "boolean",
        "mutable": false,
        "complexType": {
          "original": "boolean",
          "resolved": "boolean",
          "references": {}
        },
        "required": false,
        "optional": false,
        "docs": {
          "tags": [],
          "text": ""
        },
        "attribute": "disabled",
        "reflect": false,
        "defaultValue": "false"
      },
      "error": {
        "type": "boolean",
        "mutable": true,
        "complexType": {
          "original": "boolean",
          "resolved": "boolean",
          "references": {}
        },
        "required": false,
        "optional": false,
        "docs": {
          "tags": [],
          "text": ""
        },
        "attribute": "error",
        "reflect": true,
        "defaultValue": "false"
      },
      "filename": {
        "type": "string",
        "mutable": true,
        "complexType": {
          "original": "string",
          "resolved": "string",
          "references": {}
        },
        "required": false,
        "optional": false,
        "docs": {
          "tags": [],
          "text": "The filename to display. This is useful for previously uploaded files.  If provided, the Remove File button is shown."
        },
        "attribute": "filename",
        "reflect": true
      },
      "inputId": {
        "type": "string",
        "mutable": false,
        "complexType": {
          "original": "string",
          "resolved": "string",
          "references": {}
        },
        "required": false,
        "optional": false,
        "docs": {
          "tags": [],
          "text": "The `id` attribute to apply to the input element"
        },
        "attribute": "input-id",
        "reflect": false
      },
      "label": {
        "type": "string",
        "mutable": false,
        "complexType": {
          "original": "string",
          "resolved": "string",
          "references": {}
        },
        "required": false,
        "optional": false,
        "docs": {
          "tags": [],
          "text": "Text for the label element"
        },
        "attribute": "label",
        "reflect": false
      },
      "name": {
        "type": "string",
        "mutable": false,
        "complexType": {
          "original": "string",
          "resolved": "string",
          "references": {}
        },
        "required": false,
        "optional": false,
        "docs": {
          "tags": [],
          "text": "The `name` attribute for the input element"
        },
        "attribute": "name",
        "reflect": false
      },
      "placeholder": {
        "type": "string",
        "mutable": false,
        "complexType": {
          "original": "string",
          "resolved": "string",
          "references": {}
        },
        "required": false,
        "optional": false,
        "docs": {
          "tags": [],
          "text": "The text to show when no file is selected"
        },
        "attribute": "placeholder",
        "reflect": false,
        "defaultValue": "'No file chosen'"
      },
      "readonly": {
        "type": "boolean",
        "mutable": false,
        "complexType": {
          "original": "boolean",
          "resolved": "boolean",
          "references": {}
        },
        "required": false,
        "optional": false,
        "docs": {
          "tags": [],
          "text": ""
        },
        "attribute": "readonly",
        "reflect": false,
        "defaultValue": "false"
      },
      "thumbnailSize": {
        "type": "string",
        "mutable": false,
        "complexType": {
          "original": "'cover' | 'stretch' | 'contain' | 'auto'",
          "resolved": "\"auto\" | \"contain\" | \"cover\" | \"stretch\"",
          "references": {}
        },
        "required": false,
        "optional": false,
        "docs": {
          "tags": [],
          "text": "Sets the thumbnail sizing strategy relative to the container."
        },
        "attribute": "thumbnail-size",
        "reflect": false,
        "defaultValue": "'cover'"
      },
      "thumbnailUrl": {
        "type": "string",
        "mutable": true,
        "complexType": {
          "original": "string",
          "resolved": "string",
          "references": {}
        },
        "required": false,
        "optional": false,
        "docs": {
          "tags": [],
          "text": "The URL for the thumbnail of the currently selected image.  If provided, the Remove File button will be shown."
        },
        "attribute": "thumbnail-url",
        "reflect": true
      }
    };
  }
  static get states() {
    return {
      "thumbnailDataUri": {}
    };
  }
  static get events() {
    return [{
        "method": "mxThumbnailChange",
        "name": "mxThumbnailChange",
        "bubbles": true,
        "cancelable": true,
        "composed": true,
        "docs": {
          "tags": [],
          "text": "Emits the thumbnail url as `CustomEvent.detail` whenever it changes (i.e. after generating a data URI)"
        },
        "complexType": {
          "original": "string",
          "resolved": "string",
          "references": {}
        }
      }];
  }
  static get methods() {
    return {
      "removeFile": {
        "complexType": {
          "signature": "() => Promise<void>",
          "parameters": [],
          "references": {
            "Promise": {
              "location": "global",
              "id": "global::Promise"
            }
          },
          "return": "Promise<void>"
        },
        "docs": {
          "text": "",
          "tags": []
        }
      },
      "selectFile": {
        "complexType": {
          "signature": "() => Promise<void>",
          "parameters": [],
          "references": {
            "Promise": {
              "location": "global",
              "id": "global::Promise"
            }
          },
          "return": "Promise<void>"
        },
        "docs": {
          "text": "",
          "tags": []
        }
      }
    };
  }
  static get elementRef() { return "element"; }
  static get watchers() {
    return [{
        "propName": "thumbnailUrl",
        "methodName": "onThumbnailUrlChange"
      }];
  }
}
//# sourceMappingURL=mx-block-file-input.js.map
