import { newSpecPage } from "@stencil/core/testing";
import { MxBlockFileInput } from "../mx-block-file-input";
describe('mx-block-file-input', () => {
  let page;
  let root;
  let input;
  let chooseFileButton;
  let filenameElement;
  beforeEach(async () => {
    page = await newSpecPage({
      components: [MxBlockFileInput],
      html: `
        <mx-block-file-input
          assistive-text="testAssistiveText"
          input-id="testId"
          label="testLabel"
          name="testName"
          placeholder="testPlaceholder"
          accept-image
        />
      `,
    });
    root = page.root;
    input = root.querySelector('input');
    chooseFileButton = root.querySelector('[data-testid="choose-file-button"]');
    filenameElement = root.querySelector('[data-testid="filename"]');
  });
  it('assigns proper value for label', async () => {
    const labelElement = root.querySelector('label');
    expect(labelElement.textContent).toContain('testLabel');
    expect(labelElement.textContent).toContain('testAssistiveText');
  });
  it('sets the input name and id', async () => {
    expect(input.getAttribute('name')).toBe('testName');
    expect(input.getAttribute('id')).toBe('testId');
  });
  it('sets the input aria-describedby', async () => {
    expect(input.getAttribute('aria-describedby')).toBe(`${root.inputId}-assistive ${root.inputId}-filename`);
  });
  it('displays placeholder when no file is selected', async () => {
    expect(filenameElement.textContent).toContain('testPlaceholder');
  });
  it('sets the accept string', async () => {
    expect(input.getAttribute('accept')).toBe('image/*');
    root.acceptPdf = true;
    await page.waitForChanges();
    expect(input.getAttribute('accept')).toBe('image/*,.pdf');
    root.accept = '.txt';
    await page.waitForChanges();
    expect(input.getAttribute('accept')).toBe('image/*,.pdf,.txt');
  });
  it('sets the input disabled', async () => {
    expect(input.getAttribute('disabled')).toBeNull();
    root.disabled = true;
    await page.waitForChanges();
    expect(input.getAttribute('disabled')).not.toBeNull();
  });
  it('sets the Choose File button attributes', async () => {
    expect(chooseFileButton.getAttribute('btn-type')).toBe('contained');
    expect(chooseFileButton.textContent).toBe('Choose File');
    root.chooseBtnType = 'outlined';
    root.chooseFileText = 'Select File';
    await page.waitForChanges();
    expect(chooseFileButton.getAttribute('btn-type')).toBe('outlined');
    expect(chooseFileButton.textContent).toBe('Select File');
  });
  it('renders the provided filename', async () => {
    root.filename = 'testFilename';
    await page.waitForChanges();
    expect(filenameElement.textContent).toContain('testFilename');
  });
  it('renders a thumbnail from the thumbnailUrl prop', async () => {
    root.thumbnailUrl = 'test.jpg';
    await page.waitForChanges();
    const thumbnail = root.querySelector('[data-testid="thumbnail"]');
    expect(thumbnail.style.backgroundImage).toBe('url(test.jpg)');
  });
  it('sets the background-size of the thumbnail based on the thumbnailSize prop', async () => {
    root.thumbnailUrl = 'test.jpg';
    await page.waitForChanges();
    const thumbnail = root.querySelector('[data-testid="thumbnail"]');
    expect(thumbnail.style.backgroundSize).toBe('cover');
    root.thumbnailSize = 'contain';
    await page.waitForChanges();
    expect(thumbnail.style.backgroundSize).toBe('contain');
    root.thumbnailSize = 'stretch';
    await page.waitForChanges();
    expect(thumbnail.style.backgroundSize).toBe('100% 100%');
  });
  it('shows the Remove File button if filename is set', async () => {
    expect(root.querySelector('[data-testid="remove-file-button"]')).toBeNull();
    root.filename = 'testFilename';
    await page.waitForChanges();
    expect(root.querySelector('[data-testid="remove-file-button"]')).not.toBeNull();
  });
  it('shows the Remove File button if thumbnailUrl is set', async () => {
    expect(root.querySelector('[data-testid="remove-file-button"]')).toBeNull();
    root.thumbnailUrl = 'testUrl';
    await page.waitForChanges();
    expect(root.querySelector('[data-testid="remove-file-button"]')).not.toBeNull();
  });
  it('hides the Remove File button if disabled', async () => {
    root.filename = 'testFilename';
    root.disabled = true;
    await page.waitForChanges();
    expect(root.querySelector('[data-testid="remove-file-button"]')).toBeNull();
  });
  it('emits an mxThumbnailChange event when thumbnailUrl is changed', async () => {
    const listener = jest.fn();
    root.addEventListener('mxThumbnailChange', listener);
    root.thumbnailUrl = 'test.jpg';
    await page.waitForChanges();
    expect(listener).toHaveBeenCalled();
    expect(listener.mock.calls[0][0].detail).toBe('test.jpg');
  });
  it('emits input, change, and mxThumbnailChange events when Remove is clicked', async () => {
    root.thumbnailUrl = 'test.jpg';
    await page.waitForChanges();
    const listener = jest.fn();
    root.addEventListener('input', listener);
    root.addEventListener('change', listener);
    root.addEventListener('mxThumbnailChange', listener);
    const removeFileButton = root.querySelector('[data-testid="remove-file-button"]');
    removeFileButton.click();
    expect(listener).toHaveBeenCalledTimes(3);
    const value = listener.mock.calls[0][0].target.value;
    const thumbnailUrl = listener.mock.calls[2][0].target.value;
    expect(value).toBeFalsy();
    expect(thumbnailUrl).toBeFalsy();
  });
  it('clicks the input when the selectFile method is called', async () => {
    const listener = jest.fn();
    input.addEventListener('click', listener);
    root.selectFile();
    await page.waitForChanges();
    expect(listener).toHaveBeenCalled();
  });
});
//# sourceMappingURL=mx-block-file-input.spec.js.map
