import { newSpecPage } from "@stencil/core/testing";
import { MxBlockInput } from "../mx-block-input";
describe('mx-block-input', () => {
  let page;
  let root;
  let input;
  let label;
  beforeEach(async () => {
    page = await newSpecPage({
      components: [MxBlockInput],
      html: `
        <mx-block-input
          assistive-text="testAssistiveText"
          colspan="2"
          inputId="testId"
          label="testLabel"
          maxlength="10"
          name="testName"
          placeholder="testPlaceholder"
          type="email"
          value="testValue"
          data-test="testData"
        >
          <mx-button slot="button">Button</mx-button>
        </mx-block-input>
      `,
    });
    root = page.root;
    input = root.querySelector('input');
    label = root.querySelector('label');
  });
  it('assigns proper value for label', async () => {
    const placeholder = root.querySelector('label');
    expect(placeholder.textContent).toContain('testLabel');
  });
  it('associates the label with the input', async () => {
    expect(label.getAttribute('htmlFor')).toBe(input.getAttribute('id'));
  });
  it('has the right name, value, placeholder and type', async () => {
    expect(input.getAttribute('name')).toBe('testName');
    expect(input.value).toBe('testValue');
    expect(input.placeholder).toBe('testPlaceholder');
    expect(input.getAttribute('type')).toBe('email');
  });
  it('updates the value prop when the input value is changed', async () => {
    input.value = 'bar';
    input.dispatchEvent(new Event('input'));
    await page.waitForChanges();
    expect(root.value).toBe('bar');
  });
  it('sets the appropriate col-span-* class', async () => {
    expect(root.classList.contains('md:col-span-2')).toBe(true);
  });
  it('should have proper assistive text', async () => {
    const assitive = label.querySelector('span');
    expect(assitive.textContent).toBe('testAssistiveText');
  });
  it('disables the input based on the disabled prop', async () => {
    expect(input.getAttribute('disabled')).toBeNull();
    root.disabled = true;
    await page.waitForChanges();
    expect(input.getAttribute('disabled')).not.toBeNull();
  });
  it('sets the inputs to read-only if the readonly prop is set', async () => {
    expect(input.getAttribute('readonly')).toBeNull();
    root.readonly = true;
    await page.waitForChanges();
    expect(input.getAttribute('readonly')).not.toBeNull();
  });
  it('sets the maxlength attribute on the input', async () => {
    expect(input.getAttribute('maxlength')).toBe('10');
  });
  it('applies any data attributes to the input element', async () => {
    expect(input.getAttribute('data-test')).toBe('testData');
  });
  it('associates assistive text to the aria-describedby attribute', async () => {
    expect(input.getAttribute('aria-describedby')).toBe(label.querySelector('span').getAttribute('id'));
  });
  it('only shows the clear button when a value is present', async () => {
    expect(root.querySelector('[data-testid="clear-button"]')).not.toBeNull();
    root.value = '';
    await page.waitForChanges();
    expect(root.querySelector('[data-testid="clear-button"]')).toBeNull();
  });
  it('clears the input when the clear button is clicked and emits the appropriate events', async () => {
    const inputListener = jest.fn();
    const mxClearListener = jest.fn();
    root.addEventListener('input', inputListener);
    root.addEventListener('mxClear', mxClearListener);
    const clearButton = root.querySelector('[data-testid="clear-button"]');
    clearButton.click();
    await page.waitForChanges();
    expect(root.value).toBe('');
    expect(inputListener).toHaveBeenCalled();
    expect(mxClearListener).toHaveBeenCalled();
  });
  it('renders the button slot content inside the grid', async () => {
    const button = root.querySelector('mx-button');
    expect(button).not.toBeNull();
    expect(button.parentElement.tagName).not.toBe('MX-BLOCK-INPUT');
  });
});
//# sourceMappingURL=mx-block-input.spec.js.map
