import { newSpecPage } from "@stencil/core/testing";
import { MxBlockSearch } from "../mx-block-search";
import { MxBlockInput } from "../../mx-block-input/mx-block-input";
describe('mx-block-search', () => {
  let page;
  let root;
  let input;
  let label;
  let searchButton;
  beforeEach(async () => {
    page = await newSpecPage({
      components: [MxBlockSearch, MxBlockInput],
      html: `
        <mx-block-search
          assistive-text="testAssistiveText"
          colspan="2"
          inputId="testId"
          label="testLabel"
          maxlength="10"
          name="testName"
          placeholder="testPlaceholder"
          value="testValue"
          data-test="testData"
        />
      `,
    });
    root = page.root;
    input = root.querySelector('input');
    label = root.querySelector('label');
    searchButton = root.querySelector('[data-testid="search-button"]');
  });
  it('assigns proper value for label', async () => {
    const placeholder = root.querySelector('label');
    expect(placeholder.textContent).toContain('testLabel');
  });
  it('associates the label with the input', async () => {
    expect(label.getAttribute('htmlFor')).toBe(input.getAttribute('id'));
  });
  it('has the right name, value, placeholder and type', async () => {
    expect(input.getAttribute('name')).toBe('testName');
    expect(input.value).toBe('testValue');
    expect(input.placeholder).toBe('testPlaceholder');
    expect(input.getAttribute('type')).toBe('search');
  });
  it('updates the value prop when the input value is changed', async () => {
    input.value = 'bar';
    input.dispatchEvent(new Event('input', { bubbles: true }));
    await page.waitForChanges();
    expect(root.value).toBe('bar');
  });
  it('should have proper assistive text', async () => {
    const assitive = label.querySelector('span');
    expect(assitive.textContent).toBe('testAssistiveText');
  });
  it('disables the input based on the disabled prop', async () => {
    expect(input.getAttribute('disabled')).toBeNull();
    root.disabled = true;
    await page.waitForChanges();
    expect(input.getAttribute('disabled')).not.toBeNull();
  });
  it('sets the inputs to read-only if the readonly prop is set', async () => {
    expect(input.getAttribute('readonly')).toBeNull();
    root.readonly = true;
    await page.waitForChanges();
    expect(input.getAttribute('readonly')).not.toBeNull();
  });
  it('sets the maxlength attribute on the input', async () => {
    expect(input.getAttribute('maxlength')).toBe('10');
  });
  it('applies any data attributes to the input element', async () => {
    expect(input.getAttribute('data-test')).toBe('testData');
  });
  it('associates assistive text to the aria-describedby attribute', async () => {
    expect(input.getAttribute('aria-describedby')).toBe(label.querySelector('span').getAttribute('id'));
  });
  it('clears the input when the clear button is clicked and emits the appropriate events', async () => {
    const inputListener = jest.fn();
    const mxClearListener = jest.fn();
    root.addEventListener('input', inputListener);
    root.addEventListener('mxClear', mxClearListener);
    const clearButton = root.querySelector('[data-testid="clear-button"]');
    clearButton.click();
    await page.waitForChanges();
    expect(root.value).toBe('');
    expect(inputListener).toHaveBeenCalled();
    expect(mxClearListener).toHaveBeenCalled();
  });
  it('only shows the search button when a value is present', async () => {
    expect(searchButton.classList.contains('hidden')).toBe(false);
    root.value = '';
    await page.waitForChanges();
    expect(searchButton.classList.contains('hidden')).toBe(true);
  });
  it('emits an mxSearch event when the search button is clicked', () => {
    const mxSearchListener = jest.fn();
    root.addEventListener('mxSearch', mxSearchListener);
    searchButton.click();
    expect(mxSearchListener).toHaveBeenCalled();
  });
});
//# sourceMappingURL=mx-block-search.spec.js.map
