import "../../../utils/matchMedia.mock";
import { newSpecPage } from "@stencil/core/testing";
import { MxBlockSelect } from "../mx-block-select";
import { MxMenu } from "../../mx-menu/mx-menu";
import { MxMenuItem } from "../../mx-menu-item/mx-menu-item";
describe('mx-block-select', () => {
  let page;
  let root;
  let dropdownWrapper;
  let input;
  let label;
  let menuItems;
  beforeEach(async () => {
    page = await newSpecPage({
      components: [MxBlockSelect, MxMenu, MxMenuItem],
      html: `
        <mx-block-select
          assistive-text="testAssistiveText"
          colspan="2"
          inputId="testId"
          label="testLabel"
          name="testName"
          placeholder="testPlaceholder"
          type="email"
          value="TestValue2"
          data-test="testData"
        >
          <mx-menu-item>TestValue1</mx-menu-item>
          <mx-menu-item>TestValue2</mx-menu-item>
          <mx-menu-item>TestValue3</mx-menu-item>
        </mx-block-select>
      `,
    });
    root = page.root;
    dropdownWrapper = root.querySelector('div');
    input = root.querySelector('input');
    label = root.querySelector('label');
    menuItems = root.querySelectorAll('mx-menu-item');
    await page.waitForChanges();
  });
  it('assigns proper value for label', async () => {
    const placeholder = root.querySelector('label');
    expect(placeholder.textContent).toContain('testLabel');
  });
  it('associates the label with the input', async () => {
    expect(label.getAttribute('htmlFor')).toBe(input.getAttribute('id'));
  });
  it('has the right name, value, placeholder and type', async () => {
    expect(input.getAttribute('name')).toBe('testName');
    expect(input.value).toBe('TestValue2');
    expect(input.placeholder).toBe('testPlaceholder');
    expect(input.getAttribute('type')).toBe('email');
  });
  it('updates the value prop when the input value is changed', async () => {
    input.value = 'bar';
    input.dispatchEvent(new Event('input'));
    await page.waitForChanges();
    expect(root.value).toBe('bar');
  });
  it('should have proper assistive text', async () => {
    const assitive = label.querySelector('span');
    expect(assitive.textContent).toBe('testAssistiveText');
  });
  it('disables the input based on the disabled prop', async () => {
    expect(input.getAttribute('disabled')).toBeNull();
    root.disabled = true;
    await page.waitForChanges();
    expect(input.getAttribute('disabled')).not.toBeNull();
  });
  it('applies any data attributes to the input element', async () => {
    expect(input.getAttribute('data-test')).toBe('testData');
  });
  it('associates assistive text to the aria-describedby attribute', async () => {
    expect(input.getAttribute('aria-describedby')).toBe(label.querySelector('span').getAttribute('id'));
  });
  it('disables the input and removes the readonly attribute if the disabled prop is set', async () => {
    root.disabled = true;
    await page.waitForChanges();
    expect(input.getAttribute('disabled')).not.toBe(null);
    expect(input.getAttribute('readonly')).toBe(null);
  });
  it('does not disable the input if the readonly prop is set', async () => {
    root.readonly = true;
    await page.waitForChanges();
    expect(input.getAttribute('disabled')).toBe(null);
    expect(input.getAttribute('readonly')).not.toBe(null);
  });
  it('updates the value when a menu item is clicked', async () => {
    menuItems[0].click();
    await page.waitForChanges();
    expect(root.value).toBe('TestValue1');
  });
  it('emits an input event when the selected value changes', async () => {
    const listener = jest.fn();
    root.addEventListener('input', listener);
    menuItems[2].click();
    await page.waitForChanges();
    expect(listener).toHaveBeenCalled();
  });
  it('sets an error state on the wrapper class', async () => {
    root.error = true;
    await page.waitForChanges();
    expect(dropdownWrapper.getAttribute('class')).toContain('error');
  });
});
//# sourceMappingURL=mx-block-select.spec.js.map
