import { newSpecPage } from "@stencil/core/testing";
import { MxBlockTextarea } from "../mx-block-textarea";
describe('mx-block-textarea', () => {
  let page;
  let root;
  let textarea;
  let label;
  beforeEach(async () => {
    page = await newSpecPage({
      components: [MxBlockTextarea],
      html: `
        <mx-block-textarea
          assistive-text="testAssistiveText"
          inputId="testId"
          label="testLabel"
          maxlength="10"
          name="testName"
          placeholder="testPlaceholder"
          value="testValue"
          data-test="testData"
        >
          <mx-button slot="button">Button</mx-button>
        </mx-block-textarea>
      `,
    });
    root = page.root;
    textarea = root.querySelector('textarea');
    label = root.querySelector('label');
  });
  it('assigns proper value for label', async () => {
    const placeholder = root.querySelector('label');
    expect(placeholder.textContent).toContain('testLabel');
  });
  it('associates the label with the textarea', async () => {
    expect(label.getAttribute('htmlFor')).toBe(textarea.getAttribute('id'));
  });
  it('has the right name, value, placeholder and type', async () => {
    expect(textarea.getAttribute('name')).toBe('testName');
    expect(textarea.value).toBe('testValue');
    expect(textarea.placeholder).toBe('testPlaceholder');
  });
  it('updates the value prop when the textarea value is changed', async () => {
    textarea.value = 'bar';
    textarea.dispatchEvent(new Event('input'));
    await page.waitForChanges();
    expect(root.value).toBe('bar');
  });
  it('should have proper assistive text', async () => {
    const assitive = label.querySelector('span');
    expect(assitive.textContent).toBe('testAssistiveText');
  });
  it('disables the textarea based on the disabled prop', async () => {
    expect(textarea.getAttribute('disabled')).toBeNull();
    root.disabled = true;
    await page.waitForChanges();
    expect(textarea.getAttribute('disabled')).not.toBeNull();
  });
  it('sets the textarea to read-only if the readonly prop is set', async () => {
    expect(textarea.getAttribute('readonly')).toBeNull();
    root.readonly = true;
    await page.waitForChanges();
    expect(textarea.getAttribute('readonly')).not.toBeNull();
  });
  it('sets the maxlength attribute on the textarea', async () => {
    expect(textarea.getAttribute('maxlength')).toBe('10');
  });
  it('applies any data attributes to the textarea element', async () => {
    expect(textarea.getAttribute('data-test')).toBe('testData');
  });
  it('associates assistive text to the aria-describedby attribute', async () => {
    expect(textarea.getAttribute('aria-describedby')).toBe(label.querySelector('span').getAttribute('id'));
  });
});
//# sourceMappingURL=mx-block-textarea.js.map
