import { newSpecPage } from "@stencil/core/testing";
import { MxButton } from "../mx-button";
describe('mx-button', () => {
  let page;
  let root;
  let btn;
  beforeEach(async () => {
    page = await newSpecPage({
      components: [MxButton],
      html: `
      <mx-button
        el-aria-described-by="aria-describedby"
        el-aria-label="aria label"
        el-aria-labelled-by="aria-labelledby"
        el-aria-pressed="true"
        icon="ph-apple-logo"
        btn-type="contained"
        value="foo"
        form="bar"
        formaction="/foo"
        data-admin--a--b-c="test"
      >
        button
      </mx-button>`,
    });
    root = page.root;
    btn = root.querySelector('button');
  });
  it('renders a "contained" button', async () => {
    expect(btn.getAttribute('class')).toContain('contained');
  });
  it('has the correct default type', async () => {
    expect(btn.getAttribute('type')).toBe('button');
  });
  it('has the correct inner text', async () => {
    const { innerText } = btn;
    expect(innerText.trim()).toBe('button');
  });
  it('has the correct value, form, and formaction', async () => {
    expect(btn.getAttribute('value')).toBe('foo');
    expect(btn.getAttribute('form')).toBe('bar');
    expect(btn.getAttribute('formaction')).toBe('/foo');
  });
  it('has a left icon', async () => {
    const icon = root.querySelector('button i');
    expect(icon).not.toBeNull();
  });
  it('has a min-height of 36px', async () => {
    expect(btn.getAttribute('class')).toContain('min-h-36');
  });
  it('has a min-height of 30px when small is set', async () => {
    root.small = true;
    await page.waitForChanges();
    expect(btn.getAttribute('class')).toContain('min-h-30');
  });
  it('has the hug class when hug is set', async () => {
    root.hug = true;
    await page.waitForChanges();
    expect(btn.getAttribute('class')).toContain('hug');
  });
  it('uses the elAria* props for the aria attributes', async () => {
    expect(btn.getAttribute('aria-label')).toBe('aria label');
    expect(btn.getAttribute('aria-describedby')).toBe('aria-describedby');
    expect(btn.getAttribute('aria-labelledby')).toBe('aria-labelledby');
    expect(btn.getAttribute('aria-pressed')).toBe('true');
  });
  it('applies any data attributes to the button element', async () => {
    expect(btn.getAttribute('data-admin--a--b-c')).toBe('test');
  });
});
describe('mx-button as disabled', () => {
  let page;
  let root;
  beforeEach(async () => {
    page = await newSpecPage({
      components: [MxButton],
      html: `<mx-button btn-type="contained" value="foo" disabled>button</mx-button>`,
    });
    root = page.root;
  });
  it('is a disabled button', async () => {
    const btn = root.querySelector('button');
    expect(btn.disabled).not.toBeNull();
  });
});
describe('mx-button as XL and full', () => {
  let page;
  let root;
  beforeEach(async () => {
    page = await newSpecPage({
      components: [MxButton],
      html: `<mx-button btn-type="contained" value="foo" xl full>button</mx-button>`,
    });
    root = page.root;
  });
  it('is a flex container and is 48px in min-height', async () => {
    const btn = root.querySelector('button');
    expect(root.getAttribute('class')).toContain('flex');
    expect(btn.getAttribute('class')).toContain('min-h-48');
  });
});
describe('mx-button as outlined', () => {
  let page;
  let root;
  beforeEach(async () => {
    page = await newSpecPage({
      components: [MxButton],
      html: `<mx-button btn-type="outlined" value="foo">button</mx-button>`,
    });
    root = page.root;
  });
  it('is an "outlined" button', async () => {
    const btn = root.querySelector('button');
    expect(btn.getAttribute('class')).toContain('border');
  });
});
describe('mx-button as a simple button', () => {
  let page;
  let root;
  beforeEach(async () => {
    page = await newSpecPage({
      components: [MxButton],
      html: `<mx-button btn-type="simple" value="foo" dropdown>button</mx-button>`,
    });
    root = page.root;
  });
  it('is an "outlined" button', async () => {
    const btn = root.querySelector('button');
    expect(btn.getAttribute('class')).toContain('border');
  });
  it('has a chevron icon', async () => {
    const icon = root.querySelector('button [data-testid="chevron"]');
    expect(icon).not.toBeNull();
  });
});
describe('mx-button as a text button', () => {
  let page;
  let root;
  beforeEach(async () => {
    page = await newSpecPage({
      components: [MxButton],
      html: `<mx-button btn-type="text" value="foo" dropdown>button</mx-button>`,
    });
    root = page.root;
  });
  it('is a "text" button', async () => {
    const btn = root.querySelector('button');
    expect(btn.getAttribute('class')).toContain('text');
  });
  it('has a chevron icon', async () => {
    const icon = root.querySelector('button [data-testid="chevron"]');
    expect(icon).not.toBeNull();
  });
});
describe('mx-button as an anchor tag', () => {
  let page;
  let root;
  beforeEach(async () => {
    page = await newSpecPage({
      components: [MxButton],
      html: `<mx-button href="https://google./com" target="_blank" btn-type="text" value="foo" data-admin--a--b-c="test">button</mx-button>`,
    });
    root = page.root;
  });
  it('is an anchor element', async () => {
    const btn = root.querySelector('a');
    expect(btn).not.toBeNull();
    expect(btn.getAttribute('target')).toBeDefined();
  });
  it('applies any data attributes to the <a> element', async () => {
    const btn = root.querySelector('a');
    expect(btn.getAttribute('data-admin--a--b-c')).toBe('test');
  });
});
//# sourceMappingURL=mx-button.spec.js.map
