import { Host, h } from "@stencil/core";
import { expand, collapse } from "../../utils/transitions";
import { uuidv4 } from "../../utils/utils";
export class MxMenuAccordion {
  constructor() {
    this.isClosing = false;
    this.uuid = uuidv4();
    this.anchorEl = undefined;
    this.isOpen = false;
  }
  onClick(e) {
    const anchorElWasClicked = this.anchorEl && this.anchorEl.contains(e.target);
    if (!anchorElWasClicked)
      return;
    e.preventDefault();
    this.isOpen ? this.closeMenu() : this.openMenu();
  }
  async toggleMenu() {
    return this.isOpen ? this.closeMenu : this.openMenu;
  }
  async openMenu() {
    if (this.isOpen || !this.anchorEl)
      return false;
    this.isOpen = true;
    this.mxOpen.emit();
    await new Promise(requestAnimationFrame);
    await expand(this.menuElem);
    return true;
  }
  setAnchorElAttributes() {
    if (!this.anchorEl)
      return;
    this.anchorEl.setAttribute('aria-controls', this.uuid);
    this.setAnchorAriaExpanded(this.isOpen);
  }
  async closeMenu(skipTransition = false) {
    if (!this.isOpen || this.isClosing)
      return false;
    this.isClosing = true;
    this.setAnchorAriaExpanded(false);
    this.menuItems.forEach(m => m.closeSubMenu());
    await collapse(this.menuElem, !skipTransition ? undefined : 0);
    this.isClosing = false;
    this.mxClose.emit();
    this.isOpen = false;
    return true;
  }
  componentDidLoad() {
    if (this.menuItems.length) {
      const role = this.element.querySelector('[role="option"]') ? 'listbox' : 'menu';
      this.menuElem.setAttribute('role', role);
    }
    this.setAnchorElAttributes();
  }
  componentWillUpdate() {
    this.setAnchorAriaExpanded(this.isOpen);
    const anyMenuItemHasIcon = this.menuItems.some(m => !!m.icon);
    if (anyMenuItemHasIcon) {
      this.menuItems.forEach(m => {
        if (m.icon === undefined)
          m.icon = null;
      });
    }
  }
  setAnchorAriaExpanded(isExpanded = false) {
    if (!this.anchorEl)
      return;
    this.anchorEl.setAttribute('aria-expanded', isExpanded.toString());
  }
  get menuItems() {
    return (Array.from(this.menuElem.children).filter(e => e.tagName === 'MX-MENU-ITEM') ||
      []);
  }
  get hostClass() {
    let str = 'mx-menu-accordion block z-50';
    if (!this.isOpen)
      str += ' invisible h-0 overflow-hidden';
    return str;
  }
  render() {
    return (h(Host, { class: this.hostClass }, h("div", { id: this.uuid, ref: el => (this.menuElem = el), class: "flex flex-col overflow-hidden rounded-b pl-12" }, h("slot", null))));
  }
  static get is() { return "mx-menu-accordion"; }
  static get properties() {
    return {
      "anchorEl": {
        "type": "unknown",
        "mutable": false,
        "complexType": {
          "original": "HTMLElement",
          "resolved": "HTMLElement",
          "references": {
            "HTMLElement": {
              "location": "global",
              "id": "global::HTMLElement"
            }
          }
        },
        "required": false,
        "optional": false,
        "docs": {
          "tags": [],
          "text": "The element to which the menu's position will be anchored"
        }
      },
      "isOpen": {
        "type": "boolean",
        "mutable": true,
        "complexType": {
          "original": "boolean",
          "resolved": "boolean",
          "references": {}
        },
        "required": false,
        "optional": false,
        "docs": {
          "tags": [],
          "text": ""
        },
        "attribute": "is-open",
        "reflect": true,
        "defaultValue": "false"
      }
    };
  }
  static get events() {
    return [{
        "method": "mxClose",
        "name": "mxClose",
        "bubbles": true,
        "cancelable": true,
        "composed": true,
        "docs": {
          "tags": [],
          "text": "Emitted when the menu closes."
        },
        "complexType": {
          "original": "void",
          "resolved": "void",
          "references": {}
        }
      }, {
        "method": "mxOpen",
        "name": "mxOpen",
        "bubbles": true,
        "cancelable": true,
        "composed": true,
        "docs": {
          "tags": [],
          "text": "Emitted when the menu opens."
        },
        "complexType": {
          "original": "void",
          "resolved": "void",
          "references": {}
        }
      }];
  }
  static get methods() {
    return {
      "toggleMenu": {
        "complexType": {
          "signature": "() => Promise<(skipTransition?: boolean) => Promise<boolean>>",
          "parameters": [],
          "references": {
            "Promise": {
              "location": "global",
              "id": "global::Promise"
            }
          },
          "return": "Promise<(skipTransition?: boolean) => Promise<boolean>>"
        },
        "docs": {
          "text": "Collapse/expand the accordion menu.",
          "tags": []
        }
      },
      "openMenu": {
        "complexType": {
          "signature": "() => Promise<boolean>",
          "parameters": [],
          "references": {
            "Promise": {
              "location": "global",
              "id": "global::Promise"
            }
          },
          "return": "Promise<boolean>"
        },
        "docs": {
          "text": "Open the menu.  Returns a promise that resolves to false if the menu was already open.",
          "tags": []
        }
      },
      "closeMenu": {
        "complexType": {
          "signature": "(skipTransition?: boolean) => Promise<boolean>",
          "parameters": [{
              "tags": [],
              "text": ""
            }],
          "references": {
            "Promise": {
              "location": "global",
              "id": "global::Promise"
            }
          },
          "return": "Promise<boolean>"
        },
        "docs": {
          "text": "Close the menu.  Returns a promise that resolves to false if the menu was already closed.",
          "tags": []
        }
      }
    };
  }
  static get elementRef() { return "element"; }
  static get watchers() {
    return [{
        "propName": "anchorEl",
        "methodName": "setAnchorElAttributes"
      }];
  }
  static get listeners() {
    return [{
        "name": "click",
        "method": "onClick",
        "target": "document",
        "capture": true,
        "passive": false
      }];
  }
}
//# sourceMappingURL=mx-menu-accordion.js.map
