import { Host, h } from "@stencil/core";
import { minWidthSync, MinWidths } from "../../utils/minWidthSync";
export class MxMenuItem {
  constructor() {
    this.hasParentLink = false;
    this.checked = false;
    this.disabled = false;
    this.icon = undefined;
    this.label = undefined;
    this.subtitle = undefined;
    this.multiSelect = false;
    this.selected = false;
    this.minWidths = new MinWidths();
    this.accordion = undefined;
  }
  onMouseEnter() {
    this.closeSiblingSubMenus();
    this.focusMenuItem();
    if (this.submenu) {
      clearTimeout(this.submenuDelayTimeout);
      this.submenuDelayTimeout = setTimeout(this.openSubMenu.bind(this), 150);
    }
  }
  onMouseLeave() {
    clearTimeout(this.submenuDelayTimeout);
    if (!this.disabled)
      document.activeElement.blur();
  }
  onFocus() {
    this.closeSiblingSubMenus();
  }
  onKeyDown(e) {
    if (this.accordion)
      return this.onKeyDownAccordion(e);
    if (this.submenu)
      return this.onKeyDownSubMenu(e);
    const clickKeys = ['Enter'];
    if (this.multiSelect)
      clickKeys.push(' ');
    if (clickKeys.includes(e.key)) {
      e.preventDefault();
      e.stopPropagation();
      document.activeElement.click();
    }
  }
  componentWillLoad() {
    this.submenu = this.element.querySelector('[slot="submenu"]');
    this.accordion = this.element.querySelector('[slot="accordion"]');
  }
  componentDidLoad() {
    if (this.accordion)
      this.accordion.anchorEl = this.menuItemElem;
  }
  connectedCallback() {
    const parentLink = this.element.closest('a');
    if (parentLink) {
      this.hasParentLink = true;
      parentLink.setAttribute('role', 'menuitem');
    }
    else {
      this.role = this.element.closest('mx-dropdown-menu') ? 'option' : 'menuitem';
    }
    minWidthSync.subscribeComponent(this);
  }
  disconnectedCallback() {
    minWidthSync.unsubscribeComponent(this);
  }
  async closeAccordion(skipTransition = false) {
    if (this.accordion) {
      return await this.accordion.closeMenu(skipTransition);
    }
  }
  async closeSubMenu() {
    if (this.submenu) {
      clearTimeout(this.submenuDelayTimeout);
      return await this.submenu.closeMenu();
    }
  }
  async getValue() {
    return this.slotWrapper && this.slotWrapper.innerText.trim();
  }
  async focusMenuItem() {
    if (this.disabled)
      return;
    if (this.multiSelect) {
      const label = this.menuItemElem.querySelector('mx-checkbox label');
      label && label.focus();
    }
    else {
      this.menuItemElem.focus();
    }
  }
  onKeyDownAccordion(e) {
    if (!['Enter', ' ', 'ArrowRight', 'ArrowLeft'].includes(e.key))
      return;
    e.preventDefault();
    e.stopPropagation();
    if (e.key === 'ArrowRight') {
      this.accordion.openMenu();
    }
    else if (e.key === 'ArrowLeft') {
      this.accordion.closeMenu();
      this.focusMenuItem();
    }
    else {
      document.activeElement.click();
    }
  }
  async onKeyDownSubMenu(e) {
    if (['Enter', ' ', 'ArrowRight'].includes(e.key)) {
      e.preventDefault();
      e.stopPropagation();
      const didOpen = await this.openSubMenu();
      if (didOpen) {
        const firstMenuItem = this.element.querySelector('mx-menu-item:not(:disabled)');
        firstMenuItem && firstMenuItem.focusMenuItem();
      }
    }
    else if (e.key === 'ArrowLeft') {
      e.preventDefault();
      e.stopPropagation();
      const didClose = await this.closeSubMenu();
      if (didClose) {
        this.focusMenuItem();
      }
      else {
        this.element.parentElement.dispatchEvent(new KeyboardEvent(e.type, e));
      }
    }
  }
  closeSiblingSubMenus() {
    const siblingMenuItems = Array.from(this.element.parentElement.children).filter(e => e !== this.element && e.tagName === 'MX-MENU-ITEM');
    siblingMenuItems.forEach((m) => m.closeSubMenu());
  }
  openSubMenu() {
    if (this.submenu) {
      this.submenu.placement = 'right-start';
      this.submenu.anchorEl = this.menuItemElem;
      return this.submenu.openMenu();
    }
  }
  onClick(e) {
    if (this.disabled || !!this.submenu || !!this.accordion) {
      e.stopPropagation();
      e.preventDefault();
      return;
    }
    if (!this.multiSelect)
      this.mxClick.emit(e);
  }
  get checkboxLabel() {
    return (this.slotWrapper || this.element).innerText;
  }
  render() {
    return (h(Host, { role: "none", class: 'mx-menu-item block' + (this.submenu ? ' has-submenu' : '') }, h("div", { ref: el => (this.menuItemElem = el), role: this.role, "aria-checked": this.role === 'option' ? (this.checked ? 'true' : 'false') : null, "aria-disabled": this.disabled ? 'true' : null, "aria-haspopup": this.submenu ? 'true' : null, "aria-selected": this.selected ? 'true' : null, tabindex: this.disabled || this.multiSelect || this.hasParentLink ? '-1' : '0', class: "block w-full cursor-pointer select-none text-4 outline-none rounded disabled:cursor-auto", onClick: this.onClick.bind(this) }, this.label && (h("p", { class: "item-label flex items-end py-0 pl-14 pr-10 my-0 h-18 uppercase subtitle5" }, h("span", { class: "block -mb-4" }, this.label))), h("div", { class: 'flex items-center w-full justify-between pl-14 pr-10 h-40 whitespace-nowrap' +
        (this.multiSelect ? ' hidden' : '') }, h("div", { class: "flex items-center w-full h-full" }, this.icon !== undefined && (h("i", { class: 'inline-flex items-center justify-center text-1 w-20 mr-8 ' + this.icon })), h("span", { ref: el => (this.slotWrapper = el), class: "truncate" }, h("slot", null)), h("span", { class: "ml-auto" }, h("slot", { name: "right" }))), this.checked && !this.multiSelect && h("i", { class: "check mds-check text-icon ml-14", "data-testid": "check" }), !!this.accordion && (h("i", { class: "accordion-arrow mds-chevron-right text-icon transform transition-transform", "data-testid": "accordion-arrow" })), !!this.submenu && (h("i", { class: "mds-arrow-triangle-down text-icon transform -rotate-90", "data-testid": "arrow" }))), this.subtitle && (h("p", { class: "item-subtitle flex items-start py-0 pl-14 pr-10 my-0 h-16 caption2" }, h("span", { class: "block -mt-4 truncate" }, this.subtitle))), this.multiSelect && (h("mx-checkbox", { class: "flex items-stretch w-full overflow-hidden h-48 sm:h-32", "label-class": "pl-14 pr-16", checked: this.checked, "label-name": this.checkboxLabel, "label-left": !this.minWidths.sm }))), h("slot", { name: "accordion" }), h("slot", { name: "submenu" })));
  }
  static get is() { return "mx-menu-item"; }
  static get properties() {
    return {
      "checked": {
        "type": "boolean",
        "mutable": false,
        "complexType": {
          "original": "boolean",
          "resolved": "boolean",
          "references": {}
        },
        "required": false,
        "optional": false,
        "docs": {
          "tags": [],
          "text": "If `multiSelect` is false, this will render a checkmark on the right side of the menu item.  If both `multiSelect` and `checked` are `true`, then the rendered multi-select checkbox will be checked."
        },
        "attribute": "checked",
        "reflect": false,
        "defaultValue": "false"
      },
      "disabled": {
        "type": "boolean",
        "mutable": false,
        "complexType": {
          "original": "boolean",
          "resolved": "boolean",
          "references": {}
        },
        "required": false,
        "optional": false,
        "docs": {
          "tags": [],
          "text": ""
        },
        "attribute": "disabled",
        "reflect": false,
        "defaultValue": "false"
      },
      "icon": {
        "type": "string",
        "mutable": false,
        "complexType": {
          "original": "string",
          "resolved": "string",
          "references": {}
        },
        "required": false,
        "optional": false,
        "docs": {
          "tags": [],
          "text": "The class name of the icon to display on the left. This is sometimes automatically set to `null` to add an empty icon for alignment purposes (when a sibling menu item has an icon)."
        },
        "attribute": "icon",
        "reflect": false
      },
      "label": {
        "type": "string",
        "mutable": false,
        "complexType": {
          "original": "string",
          "resolved": "string",
          "references": {}
        },
        "required": false,
        "optional": false,
        "docs": {
          "tags": [],
          "text": "A label to display above the menu item"
        },
        "attribute": "label",
        "reflect": false
      },
      "subtitle": {
        "type": "string",
        "mutable": false,
        "complexType": {
          "original": "string",
          "resolved": "string",
          "references": {}
        },
        "required": false,
        "optional": false,
        "docs": {
          "tags": [],
          "text": "A subtitle to display below the menu item text"
        },
        "attribute": "subtitle",
        "reflect": false
      },
      "multiSelect": {
        "type": "boolean",
        "mutable": false,
        "complexType": {
          "original": "boolean",
          "resolved": "boolean",
          "references": {}
        },
        "required": false,
        "optional": false,
        "docs": {
          "tags": [],
          "text": "Render a checkbox as part of the menu item.  On small screens, the checkbox will appear on the left; otherwise, it will be on the right."
        },
        "attribute": "multi-select",
        "reflect": false,
        "defaultValue": "false"
      },
      "selected": {
        "type": "boolean",
        "mutable": false,
        "complexType": {
          "original": "boolean",
          "resolved": "boolean",
          "references": {}
        },
        "required": false,
        "optional": false,
        "docs": {
          "tags": [],
          "text": "This is automatically set by a parent Dropdown Menu."
        },
        "attribute": "selected",
        "reflect": false,
        "defaultValue": "false"
      }
    };
  }
  static get states() {
    return {
      "minWidths": {},
      "accordion": {}
    };
  }
  static get events() {
    return [{
        "method": "mxClick",
        "name": "mxClick",
        "bubbles": true,
        "cancelable": true,
        "composed": true,
        "docs": {
          "tags": [],
          "text": "Fired when an enabled menu item without a submenu is clicked. Used interally to close all ancestor menus."
        },
        "complexType": {
          "original": "MouseEvent",
          "resolved": "MouseEvent",
          "references": {
            "MouseEvent": {
              "location": "global",
              "id": "global::MouseEvent"
            }
          }
        }
      }];
  }
  static get methods() {
    return {
      "closeAccordion": {
        "complexType": {
          "signature": "(skipTransition?: boolean) => Promise<boolean>",
          "parameters": [{
              "tags": [],
              "text": ""
            }],
          "references": {
            "Promise": {
              "location": "global",
              "id": "global::Promise"
            }
          },
          "return": "Promise<boolean>"
        },
        "docs": {
          "text": "Close the item's accordion.",
          "tags": []
        }
      },
      "closeSubMenu": {
        "complexType": {
          "signature": "() => Promise<boolean>",
          "parameters": [],
          "references": {
            "Promise": {
              "location": "global",
              "id": "global::Promise"
            }
          },
          "return": "Promise<boolean>"
        },
        "docs": {
          "text": "Close the item's submenu.",
          "tags": []
        }
      },
      "getValue": {
        "complexType": {
          "signature": "() => Promise<string>",
          "parameters": [],
          "references": {
            "Promise": {
              "location": "global",
              "id": "global::Promise"
            }
          },
          "return": "Promise<string>"
        },
        "docs": {
          "text": "Returns the menu item inner text (excluding any label or subtitle)",
          "tags": []
        }
      },
      "focusMenuItem": {
        "complexType": {
          "signature": "() => Promise<void>",
          "parameters": [],
          "references": {
            "Promise": {
              "location": "global",
              "id": "global::Promise"
            },
            "HTMLElement": {
              "location": "global",
              "id": "global::HTMLElement"
            }
          },
          "return": "Promise<void>"
        },
        "docs": {
          "text": "Focuses the menu item.",
          "tags": []
        }
      }
    };
  }
  static get elementRef() { return "element"; }
  static get listeners() {
    return [{
        "name": "mouseenter",
        "method": "onMouseEnter",
        "target": undefined,
        "capture": false,
        "passive": true
      }, {
        "name": "mouseleave",
        "method": "onMouseLeave",
        "target": undefined,
        "capture": false,
        "passive": true
      }, {
        "name": "focus",
        "method": "onFocus",
        "target": undefined,
        "capture": false,
        "passive": false
      }, {
        "name": "keydown",
        "method": "onKeyDown",
        "target": undefined,
        "capture": false,
        "passive": false
      }];
  }
}
//# sourceMappingURL=mx-menu-item.js.map
