import { Host, h } from "@stencil/core";
import { createPopover, convertPlacementToOrigin, } from "../../utils/popover";
import { fadeScaleIn, fadeOut } from "../../utils/transitions";
import { uuidv4 } from "../../utils/utils";
export class MxMenu {
  constructor() {
    this.isClosing = false;
    this.uuid = uuidv4();
    this.anchorEl = undefined;
    this.autocompleteOnly = false;
    this.triggerEl = undefined;
    this.offset = undefined;
    this.placement = 'bottom-start';
    this.isOpen = false;
    this.squared = false;
  }
  onMenuItemClick() {
    this.closeMenu();
  }
  onClick(e) {
    const triggerEl = this.triggerEl || this.anchorEl;
    const triggerElWasClicked = triggerEl && triggerEl.contains(e.target);
    if (triggerElWasClicked)
      e.preventDefault();
    if (!this.isOpen && triggerElWasClicked) {
      this.openMenu();
      e.preventDefault();
    }
    else if (this.isOpen && this.element && !this.element.contains(e.target)) {
      if (this.isSubMenu && triggerElWasClicked)
        return;
      if (this.inputEl && this.inputEl.contains(e.target))
        return;
      this.closeMenu();
    }
  }
  onFocus(e) {
    if (!this.anchorEl)
      return;
    if (this.isOpen && !this.anchorEl.contains(e.target) && !this.element.contains(e.target))
      this.closeMenu();
  }
  onDocumentKeyDown(e) {
    const isFocused = (el) => el && el.contains(e.target);
    const enabledMenuItems = this.menuItems.filter(m => !m.disabled);
    if (this.autocompleteOnly && this.inputEl && this.isOpen && e.key === 'Enter' && !isFocused(this.element)) {
      e.preventDefault();
      enabledMenuItems.length && enabledMenuItems[0].click();
      this.onMenuItemClick();
      return;
    }
    const openKeys = ['Enter'];
    if (!this.inputEl)
      openKeys.push(' ');
    if (openKeys.includes(e.key) && this.anchorEl && isFocused(this.anchorEl)) {
      (this.triggerEl || this.anchorEl).click();
      e.preventDefault();
      e.stopPropagation();
      return;
    }
    if (!this.isOpen && this.inputEl && isFocused(this.inputEl)) {
      if (['Escape', 'Tab'].includes(e.key))
        return;
      (this.triggerEl || this.anchorEl).click();
    }
    if (!this.isOpen)
      return;
    const shouldEditInput = e.key.length === 1 || ['Backspace', 'Delete', 'ArrowLeft', 'ArrowRight'].includes(e.key);
    if (this.inputEl && shouldEditInput && !isFocused(this.inputEl)) {
      this.inputEl.focus();
    }
    else if (e.key === 'Escape') {
      this.closeMenu();
      e.preventDefault();
    }
    else if (['Tab', 'ArrowDown'].includes(e.key) && isFocused(this.anchorEl) && enabledMenuItems.length > 0) {
      if (e.shiftKey && e.key === 'Tab')
        return;
      if (!this.inputEl || !this.autocompleteOnly) {
        enabledMenuItems[0].focusMenuItem();
      }
      else if (this.autocompleteOnly && enabledMenuItems.length >= 2) {
        enabledMenuItems[1].focusMenuItem();
      }
      e.preventDefault();
      e.stopPropagation();
    }
    else if (e.key === 'ArrowUp' && isFocused(this.inputEl)) {
      e.preventDefault();
    }
    else if (this.inputEl && isFocused(this.inputEl)) {
      this.isOpen = !this.isOpen;
      this.isOpen = !this.isOpen;
    }
  }
  onKeydown(e) {
    if (!this.isOpen)
      return;
    if (!['ArrowDown', 'ArrowUp'].includes(e.key))
      return;
    e.preventDefault();
    e.stopPropagation();
    const tabbableMenuItems = [];
    this.menuItems.forEach(m => {
      if (m.disabled)
        return;
      tabbableMenuItems.push(m);
      const accordionMenuItems = m.querySelectorAll('[aria-expanded="true"] + mx-menu-accordion mx-menu-item');
      tabbableMenuItems.push(...Array.from(accordionMenuItems));
    });
    const focusedMenuItem = document.activeElement ? document.activeElement.closest('mx-menu-item') : null;
    const focusedIndex = tabbableMenuItems.findIndex(m => m === focusedMenuItem);
    if (e.key === 'ArrowDown' && focusedIndex !== tabbableMenuItems.length - 1) {
      tabbableMenuItems[focusedIndex + 1].focusMenuItem();
    }
    else if (e.key === 'ArrowUp' && focusedIndex > 0) {
      tabbableMenuItems[focusedIndex - 1].focusMenuItem();
    }
  }
  async openMenu() {
    if (this.isOpen || !this.anchorEl)
      return false;
    this.isOpen = true;
    this.mxOpen.emit();
    const offset = this.offset || (this.isSubMenu ? [-8, 0] : [0, 1]);
    this.popoverInstance = await createPopover(this.anchorEl, this.element, this.placement, offset);
    await fadeScaleIn(this.menuElem, undefined, convertPlacementToOrigin(this.popoverInstance.state.placement));
    return true;
  }
  setTriggerElAttributes() {
    if (!this.anchorEl && !this.triggerEl)
      return;
    (this.triggerEl || this.anchorEl).setAttribute('aria-haspopup', 'true');
    (this.triggerEl || this.anchorEl).setAttribute('aria-controls', this.uuid);
  }
  async closeMenu() {
    if (!this.isOpen || this.isClosing)
      return false;
    this.isClosing = true;
    this.menuItems.forEach(m => m.closeSubMenu());
    await fadeOut(this.menuElem);
    this.menuItems.forEach(m => m.closeAccordion(true));
    this.isClosing = false;
    this.mxClose.emit();
    this.isOpen = false;
    if (!this.popoverInstance)
      return true;
    this.popoverInstance.destroy();
    this.popoverInstance = null;
    return true;
  }
  componentDidLoad() {
    this.setInputEl();
    if (this.menuItems.length) {
      const role = this.element.querySelector('[role="option"]') ? 'listbox' : 'menu';
      this.scrollElem.setAttribute('role', role);
    }
    this.setTriggerElAttributes();
  }
  componentWillUpdate() {
    this.setInputEl();
    this.anchorEl &&
      this.anchorEl.getAttribute('role') === 'menuitem' &&
      this.anchorEl.setAttribute('aria-expanded', this.isOpen ? 'true' : 'false');
    if (this.inputEl && this.anchorEl)
      this.element.style.width = this.anchorEl.getBoundingClientRect().width + 'px';
    const anyMenuItemHasIcon = this.menuItems.some(m => !!m.icon);
    if (anyMenuItemHasIcon) {
      this.menuItems.forEach(m => {
        if (m.icon === undefined)
          m.icon = null;
      });
    }
    if (this.inputEl) {
      this.menuItems.forEach(async (m) => {
        m.selected = this.inputEl.value === (await m.getValue());
      });
    }
  }
  setInputEl() {
    if (this.anchorEl && !this.inputEl) {
      this.inputEl = this.anchorEl.querySelector('input[type="text"], input[type="search"]');
      this.inputEl && this.inputEl.setAttribute('autocomplete', 'off');
    }
  }
  get menuItems() {
    const menuItems = [];
    Array.from(this.scrollElem.children).forEach((el) => {
      if (el.tagName === 'MX-MENU-ITEM')
        menuItems.push(el);
      if (el.getAttribute('role') === 'group') {
        menuItems.push(...Array.from(el.children).filter(e => e.tagName === 'MX-MENU-ITEM'));
      }
    });
    return menuItems;
  }
  get isSubMenu() {
    return this.element.hasAttribute('slot') && this.element.getAttribute('slot') === 'submenu';
  }
  get hostClass() {
    let str = 'mx-menu block z-50 w-screen sm:w-auto';
    if (!this.isOpen)
      str += ' hidden';
    if (this.autocompleteOnly)
      str += ' autocomplete-only';
    return str;
  }
  get menuClass() {
    let str = 'flex flex-col';
    if (this.squared) {
      str += ' border shadow-drop';
    }
    else {
      str += ' shadow-9 rounded';
    }
    if (!this.isOpen)
      str += ' hidden';
    if (this.autocompleteOnly)
      str += ' autocomplete-only';
    return str;
  }
  render() {
    return (h(Host, { class: this.hostClass }, h("div", { ref: el => (this.menuElem = el), class: this.menuClass }, h("div", { id: this.uuid, ref: el => (this.scrollElem = el), class: "scroll-wrapper overflow-y-auto overflow-x-hidden overscroll-contain" }, h("slot", null)))));
  }
  static get is() { return "mx-menu"; }
  static get properties() {
    return {
      "anchorEl": {
        "type": "unknown",
        "mutable": false,
        "complexType": {
          "original": "HTMLElement",
          "resolved": "HTMLElement",
          "references": {
            "HTMLElement": {
              "location": "global",
              "id": "global::HTMLElement"
            }
          }
        },
        "required": false,
        "optional": false,
        "docs": {
          "tags": [],
          "text": "The element to which the menu's position will be anchored"
        }
      },
      "autocompleteOnly": {
        "type": "boolean",
        "mutable": false,
        "complexType": {
          "original": "boolean",
          "resolved": "boolean",
          "references": {}
        },
        "required": false,
        "optional": false,
        "docs": {
          "tags": [],
          "text": "If the anchor element contains an `input`, setting this to `true` will always select the first menu item when Enter is pressed inside the input."
        },
        "attribute": "autocomplete-only",
        "reflect": false,
        "defaultValue": "false"
      },
      "triggerEl": {
        "type": "unknown",
        "mutable": false,
        "complexType": {
          "original": "HTMLElement",
          "resolved": "HTMLElement",
          "references": {
            "HTMLElement": {
              "location": "global",
              "id": "global::HTMLElement"
            }
          }
        },
        "required": false,
        "optional": false,
        "docs": {
          "tags": [],
          "text": "The element that will open the menu when clicked.  If not provided, the `anchorEl` will be used."
        }
      },
      "offset": {
        "type": "unknown",
        "mutable": false,
        "complexType": {
          "original": "PopoverOffset",
          "resolved": "[number, number]",
          "references": {
            "PopoverOffset": {
              "location": "import",
              "path": "../../utils/popover",
              "id": "src/utils/popover.ts::PopoverOffset"
            }
          }
        },
        "required": false,
        "optional": false,
        "docs": {
          "tags": [],
          "text": "An array of offsets in pixels. The first is the \"skidding\" along the edge of the `anchorEl`.  The second is the distance from the `anchorEl`."
        }
      },
      "placement": {
        "type": "string",
        "mutable": false,
        "complexType": {
          "original": "PopoverPlacement",
          "resolved": "\"auto\" | \"auto-end\" | \"auto-start\" | \"bottom\" | \"bottom-end\" | \"bottom-start\" | \"left\" | \"left-end\" | \"left-start\" | \"right\" | \"right-end\" | \"right-start\" | \"top\" | \"top-end\" | \"top-start\"",
          "references": {
            "PopoverPlacement": {
              "location": "import",
              "path": "../../utils/popover",
              "id": "src/utils/popover.ts::PopoverPlacement"
            }
          }
        },
        "required": false,
        "optional": false,
        "docs": {
          "tags": [],
          "text": "The placement of the menu, relative to the `anchorEl`."
        },
        "attribute": "placement",
        "reflect": false,
        "defaultValue": "'bottom-start'"
      },
      "isOpen": {
        "type": "boolean",
        "mutable": true,
        "complexType": {
          "original": "boolean",
          "resolved": "boolean",
          "references": {}
        },
        "required": false,
        "optional": false,
        "docs": {
          "tags": [],
          "text": "This is set to true automatically when the `anchorEl` is clicked.  Dropdown menus read this prop internally for styling purposes."
        },
        "attribute": "is-open",
        "reflect": true,
        "defaultValue": "false"
      },
      "squared": {
        "type": "boolean",
        "mutable": false,
        "complexType": {
          "original": "boolean",
          "resolved": "boolean",
          "references": {}
        },
        "required": false,
        "optional": false,
        "docs": {
          "tags": [],
          "text": "Used for the squared variant styling."
        },
        "attribute": "squared",
        "reflect": false,
        "defaultValue": "false"
      }
    };
  }
  static get events() {
    return [{
        "method": "mxClose",
        "name": "mxClose",
        "bubbles": true,
        "cancelable": true,
        "composed": true,
        "docs": {
          "tags": [],
          "text": "Emitted when the menu closes."
        },
        "complexType": {
          "original": "void",
          "resolved": "void",
          "references": {}
        }
      }, {
        "method": "mxOpen",
        "name": "mxOpen",
        "bubbles": true,
        "cancelable": true,
        "composed": true,
        "docs": {
          "tags": [],
          "text": "Emitted when the menu opens."
        },
        "complexType": {
          "original": "void",
          "resolved": "void",
          "references": {}
        }
      }];
  }
  static get methods() {
    return {
      "openMenu": {
        "complexType": {
          "signature": "() => Promise<boolean>",
          "parameters": [],
          "references": {
            "Promise": {
              "location": "global",
              "id": "global::Promise"
            },
            "PopoverOffset": {
              "location": "import",
              "path": "../../utils/popover",
              "id": "src/utils/popover.ts::PopoverOffset"
            }
          },
          "return": "Promise<boolean>"
        },
        "docs": {
          "text": "Open the menu.  Returns a promise that resolves to false if the menu was already open.",
          "tags": []
        }
      },
      "closeMenu": {
        "complexType": {
          "signature": "() => Promise<boolean>",
          "parameters": [],
          "references": {
            "Promise": {
              "location": "global",
              "id": "global::Promise"
            }
          },
          "return": "Promise<boolean>"
        },
        "docs": {
          "text": "Close the menu.  Returns a promise that resolves to false if the menu was already closed.",
          "tags": []
        }
      }
    };
  }
  static get elementRef() { return "element"; }
  static get watchers() {
    return [{
        "propName": "anchorEl",
        "methodName": "setTriggerElAttributes"
      }, {
        "propName": "triggerEl",
        "methodName": "setTriggerElAttributes"
      }];
  }
  static get listeners() {
    return [{
        "name": "mxClick",
        "method": "onMenuItemClick",
        "target": undefined,
        "capture": false,
        "passive": false
      }, {
        "name": "click",
        "method": "onClick",
        "target": "document",
        "capture": true,
        "passive": false
      }, {
        "name": "focus",
        "method": "onFocus",
        "target": "document",
        "capture": true,
        "passive": false
      }, {
        "name": "keydown",
        "method": "onDocumentKeyDown",
        "target": "document",
        "capture": false,
        "passive": false
      }, {
        "name": "keydown",
        "method": "onKeydown",
        "target": undefined,
        "capture": false,
        "passive": false
      }];
  }
}
//# sourceMappingURL=mx-menu.js.map
