import { Host, h } from "@stencil/core";
import { moveToPortal } from "../../utils/portal";
import { fadeIn, fadeOut, fadeScaleIn } from "../../utils/transitions";
import { lockBodyScroll, unlockBodyScroll } from "../../utils/bodyScroll";
export class MxModalOverlay {
  constructor() {
    this.closeOnEscape = true;
    this.closeOnOutsideClick = true;
    this.isOpen = false;
    this.portal = true;
    this.isVisible = false;
  }
  toggleModal() {
    this.isOpen ? this.openModal() : this.closeModal();
  }
  onKeyDown(e) {
    if (!this.isOpen)
      return;
    if (e.key === 'Tab') {
      if (e.shiftKey && document.activeElement === this.firstFocusElement) {
        this.lastFocusElement.focus();
        e.preventDefault();
      }
      else if (document.activeElement === this.lastFocusElement) {
        this.firstFocusElement.focus();
        e.preventDefault();
      }
    }
  }
  onDocumentKeyDown(e) {
    if (this.isOpen && this.closeOnEscape && e.key === 'Escape')
      this.mxClose.emit();
  }
  disconnectedCallback() {
    unlockBodyScroll(this.element);
  }
  componentDidLoad() {
    if (this.isOpen)
      this.openModal();
  }
  async openModal() {
    if (this.portal)
      moveToPortal(this.element);
    lockBodyScroll(this.element);
    this.isVisible = true;
    requestAnimationFrame(async () => {
      this.getFocusElements();
      await Promise.all([fadeIn(this.backdrop, 250), fadeScaleIn(this.modal, 250)]);
    });
  }
  getFocusElements() {
    this.ancestorFocusedElement = document.activeElement;
    const isVisible = (el) => !!el.offsetParent;
    this.focusElements = Array.from(this.element.querySelectorAll('button, [href], input, select, textarea, [tabindex]:not([tabindex="-1"])')).filter(isVisible);
    if (this.focusElements.length) {
      this.firstFocusElement = this.focusElements[0];
      this.lastFocusElement = this.focusElements[this.focusElements.length - 1];
      this.focusElements[0].focus();
    }
  }
  async closeModal() {
    await Promise.all([fadeOut(this.backdrop), fadeOut(this.modal)]);
    this.isVisible = false;
    unlockBodyScroll(this.element);
    this.ancestorFocusedElement && this.ancestorFocusedElement.focus();
  }
  onBackdropClick() {
    if (this.closeOnOutsideClick)
      this.mxClose.emit();
  }
  get hostClass() {
    let str = 'mx-modal-overlay block fixed inset-0';
    if (!this.isVisible)
      str += ' hidden';
    return str;
  }
  render() {
    return (h(Host, { class: this.hostClass, "aria-modal": "true", role: "dialog" }, h("div", { ref: el => (this.backdrop = el), class: 'bg-modal-backdrop absolute inset-0 z-0' + (this.closeOnOutsideClick ? ' cursor-pointer' : ''), "data-testid": "backdrop", onClick: this.onBackdropClick.bind(this) }), h("div", { ref: el => (this.modal = el), class: "modal flex flex-col absolute inset-0 pt-60 px-10 pb-10 pointer-events-none" }, h("div", { class: "pointer-events-auto pt-10", "data-testid": "modal-content" }, h("slot", null)), h("div", { class: "absolute top-0 right-0 w-60 h-60 flex items-center justify-center" }, h("mx-icon-button", { "el-aria-label": "Close modal", close: true, class: "pointer-events-auto", "data-testid": "close-button", onClick: this.mxClose.emit })))));
  }
  static get is() { return "mx-modal-overlay"; }
  static get properties() {
    return {
      "closeOnEscape": {
        "type": "boolean",
        "mutable": false,
        "complexType": {
          "original": "boolean",
          "resolved": "boolean",
          "references": {}
        },
        "required": false,
        "optional": false,
        "docs": {
          "tags": [],
          "text": "If set to false, pressing Escape will not close the modal."
        },
        "attribute": "close-on-escape",
        "reflect": false,
        "defaultValue": "true"
      },
      "closeOnOutsideClick": {
        "type": "boolean",
        "mutable": false,
        "complexType": {
          "original": "boolean",
          "resolved": "boolean",
          "references": {}
        },
        "required": false,
        "optional": false,
        "docs": {
          "tags": [],
          "text": "If set to false, clicking the backdrop will not close the modal."
        },
        "attribute": "close-on-outside-click",
        "reflect": false,
        "defaultValue": "true"
      },
      "isOpen": {
        "type": "boolean",
        "mutable": false,
        "complexType": {
          "original": "boolean",
          "resolved": "boolean",
          "references": {}
        },
        "required": false,
        "optional": false,
        "docs": {
          "tags": [],
          "text": "Toggle the modal"
        },
        "attribute": "is-open",
        "reflect": true,
        "defaultValue": "false"
      },
      "portal": {
        "type": "boolean",
        "mutable": false,
        "complexType": {
          "original": "boolean",
          "resolved": "boolean",
          "references": {}
        },
        "required": false,
        "optional": false,
        "docs": {
          "tags": [],
          "text": "Set to false to not move the modal element to a root level \"portal\" container."
        },
        "attribute": "portal",
        "reflect": false,
        "defaultValue": "true"
      }
    };
  }
  static get states() {
    return {
      "isVisible": {}
    };
  }
  static get events() {
    return [{
        "method": "mxClose",
        "name": "mxClose",
        "bubbles": true,
        "cancelable": true,
        "composed": true,
        "docs": {
          "tags": [],
          "text": ""
        },
        "complexType": {
          "original": "any",
          "resolved": "any",
          "references": {}
        }
      }];
  }
  static get elementRef() { return "element"; }
  static get watchers() {
    return [{
        "propName": "isOpen",
        "methodName": "toggleModal"
      }];
  }
  static get listeners() {
    return [{
        "name": "keydown",
        "method": "onKeyDown",
        "target": undefined,
        "capture": false,
        "passive": false
      }, {
        "name": "keydown",
        "method": "onDocumentKeyDown",
        "target": "document",
        "capture": false,
        "passive": false
      }];
  }
}
//# sourceMappingURL=mx-modal-overlay.js.map
