import { newSpecPage } from "@stencil/core/testing";
import { MxModalOverlay } from "../mx-modal-overlay";
describe('mx-modal-overlay', () => {
  let page;
  let root;
  beforeEach(async () => {
    page = await newSpecPage({
      components: [MxModalOverlay],
      html: `
      <mx-modal-overlay>
        This is the content
      </mx-modal-overlay>
      `,
    });
    root = page.root;
    root.isOpen = true;
    await page.waitForChanges();
  });
  it('has aria-modal set to true and a role of dialog', () => {
    expect(root.getAttribute('role')).toBe('dialog');
    expect(root.getAttribute('aria-modal')).toBe('true');
  });
  it('renders the default slot content', () => {
    const content = root.querySelector('[data-testid="modal-content"]');
    expect(content.innerText).toContain('This is the content');
  });
  it('moves into the portal element by default', () => {
    expect(root.parentElement.getAttribute('class')).toContain('portal');
  });
  it('emits an mxClose event when Close is clicked', () => {
    const listener = jest.fn();
    root.addEventListener('mxClose', listener);
    root.querySelector('[data-testid="close-button"]').click();
    expect(listener).toHaveBeenCalled();
  });
  it('emits an mxClose event when the backdrop is clicked', () => {
    const listener = jest.fn();
    root.addEventListener('mxClose', listener);
    root.querySelector('[data-testid="backdrop"]').click();
    expect(listener).toHaveBeenCalled();
  });
  it('does not emit an mxClose event when the backdrop is clicked and closeOnOutsideClick is false', async () => {
    root.closeOnOutsideClick = false;
    await page.waitForChanges();
    const listener = jest.fn();
    root.addEventListener('mxClose', listener);
    root.querySelector('[data-testid="backdrop"]').click();
    expect(listener).not.toHaveBeenCalled();
  });
  it('emits an mxClose event when Escape is pressed', () => {
    const listener = jest.fn();
    root.addEventListener('mxClose', listener);
    const escape = new KeyboardEvent('keydown', { key: 'Escape' });
    document.dispatchEvent(escape);
    expect(listener).toHaveBeenCalled();
  });
  it('does not emit an mxClose event when Escape is pressed and closeOnEscape is false', async () => {
    root.closeOnEscape = false;
    await page.waitForChanges();
    const listener = jest.fn();
    root.addEventListener('mxClose', listener);
    const escape = new KeyboardEvent('keydown', { key: 'Escape' });
    document.dispatchEvent(escape);
    expect(listener).not.toHaveBeenCalled();
  });
  it('closes when isOpen is set to false', async () => {
    root.isOpen = false;
    await page.waitForChanges();
    expect(root.getAttribute('class')).toContain('hidden');
  });
});
describe('mx-modal-overlay (no portal)', () => {
  let page;
  let root;
  beforeEach(async () => {
    page = await newSpecPage({
      components: [MxModalOverlay],
      html: `
      <mx-modal-overlay portal="false">
        Content
      </mx-modal-overlay>
      `,
    });
    root = page.root;
    await page.waitForChanges();
  });
  it('does not move into the portal', () => {
    expect(root.parentElement.tagName).toBe('BODY');
  });
});
//# sourceMappingURL=mx-modal-overlay.spec.js.map
