import { Host, h } from "@stencil/core";
const MAX_PAGE_BUTTONS = 5;
export class MxPagination {
  constructor() {
    this.breakpoint = 0;
    this.page = 1;
    this.rowsPerPageOptions = [10, 25, 50, 100];
    this.rowsPerPage = 100;
    this.totalRows = undefined;
    this.disabled = false;
    this.disableNextPage = false;
    this.hideRangeAndRowsPerPage = false;
  }
  componentDidLoad() {
    if (this.rowsMenu && this.rowsMenuAnchor)
      this.rowsMenu.anchorEl = this.rowsMenuAnchor;
    this.setBreakpoint();
    this.resizeObserver = new ResizeObserver(() => this.updateResponsiveElements());
    this.resizeObserver.observe(this.element);
  }
  async setBreakpoint() {
    this.hideRangeAndRowsPerPage = false;
    await new Promise(requestAnimationFrame);
    this.breakpoint = Array.from(this.element.children).reduce((total, child) => {
      return total + child.scrollWidth;
    }, 0);
    this.updateResponsiveElements();
  }
  updateResponsiveElements() {
    this.hideRangeAndRowsPerPage = this.element.scrollWidth < this.breakpoint;
  }
  onClickFirstPage() {
    if (this.isPreviousPageDisabled)
      return;
    this.mxPageChange.emit({ page: 1, rowsPerPage: this.rowsPerPage });
  }
  onClickPreviousPage() {
    if (this.isPreviousPageDisabled)
      return;
    this.mxPageChange.emit({ page: this.page - 1, rowsPerPage: this.rowsPerPage });
  }
  onClickPageNumber(page) {
    if (page === this.page)
      return;
    this.mxPageChange.emit({ page, rowsPerPage: this.rowsPerPage });
  }
  onClickNextPage() {
    if (this.isNextPageDisabled)
      return;
    this.mxPageChange.emit({ page: this.page + 1, rowsPerPage: this.rowsPerPage });
  }
  onClickLastPage() {
    if (this.isLastPageDisabled)
      return;
    this.mxPageChange.emit({ page: this.lastPage, rowsPerPage: this.rowsPerPage });
  }
  onChangeRowsPerPage(rowsPerPage) {
    this.mxPageChange.emit({ page: 1, rowsPerPage });
  }
  get isPreviousPageDisabled() {
    return this.page === 1 || this.disabled;
  }
  get isNextPageDisabled() {
    return this.page === this.lastPage || this.disabled || this.disableNextPage;
  }
  get isFirstPageDisabled() {
    return this.page === 1 || this.disabled;
  }
  get isLastPageDisabled() {
    return this.totalRows == null || this.page === this.lastPage || this.disabled;
  }
  get lastPage() {
    if (this.totalRows === 0)
      return 1;
    if (this.totalRows == null)
      return null;
    return Math.ceil(this.totalRows / this.rowsPerPage);
  }
  get pageNumbers() {
    const length = Math.min(MAX_PAGE_BUTTONS, this.lastPage);
    if (this.page <= MAX_PAGE_BUTTONS)
      return Array.from({ length }, (_, i) => i + 1);
    if (this.page > this.lastPage - MAX_PAGE_BUTTONS) {
      return Array.from({ length }, (_, i) => this.lastPage - MAX_PAGE_BUTTONS + i + 1);
    }
    const start = this.page - Math.floor(MAX_PAGE_BUTTONS / 2);
    return Array.from({ length }, (_, i) => start + i);
  }
  get currentRange() {
    const start = this.rowsPerPage * (this.page - 1) + 1;
    const end = Math.min(this.totalRows, start + this.rowsPerPage - 1);
    return start + '–' + end;
  }
  get pageNumberGridStyle() {
    return {
      gridTemplateColumns: `repeat(${Math.min(this.lastPage, MAX_PAGE_BUTTONS)}, 1fr)`,
    };
  }
  render() {
    return (h(Host, { class: "mx-pagination relative flex items-center text-4 whitespace-nowrap select-none" }, h("nav", { "aria-label": "Page navigation", class: "flex items-center" }, h("button", { type: "button", "aria-label": "First page", disabled: this.isFirstPageDisabled, class: "underline mr-10", onClick: this.onClickFirstPage.bind(this) }, "First"), h("button", { type: "button", "aria-label": "Previous page", disabled: this.isPreviousPageDisabled, class: "pagination-icon-button text-icon", onClick: this.onClickPreviousPage.bind(this) }, h("i", { class: "mds-chevron-left" })), this.totalRows != null && (h("div", { class: "grid gap-10 ml-20", "data-testid": "page-numbers", style: this.pageNumberGridStyle }, this.pageNumbers.map(page => (h("button", { type: "button", class: "page-number-button", "aria-label": 'Page ' + page, disabled: this.page === page || this.disabled, "aria-current": this.page === page ? 'page' : null, onClick: this.onClickPageNumber.bind(this, page) }, page))))), h("button", { type: "button", "aria-label": "Next page", disabled: this.isNextPageDisabled, class: "pagination-icon-button text-icon ml-20", onClick: this.onClickNextPage.bind(this) }, h("i", { class: "mds-chevron-right" })), this.totalRows != null && (h("button", { type: "button", "aria-label": "Last page", disabled: this.isLastPageDisabled, class: "underline ml-10", onClick: this.onClickLastPage.bind(this) }, "Last"))), !this.hideRangeAndRowsPerPage && this.totalRows > 0 && (h("div", { "data-testid": "row-range", class: "text-center flex-shrink min-w-0 whitespace-nowrap pl-24" }, "Displaying ", this.currentRange, " of ", this.totalRows)), !this.hideRangeAndRowsPerPage && this.rowsPerPageOptions && this.rowsPerPageOptions.length > 1 && (h("div", { class: "flex pl-24 ml-auto" }, h("button", { type: "button", "data-testid": "rows-per-page", ref: el => (this.rowsMenuAnchor = el), class: "appearance-none flex gap-x-20 items-center cursor-pointer" }, h("span", { class: "text-small uppercase font-bold" }, "Rows per page"), h("span", null, this.rowsPerPage), h("i", { class: "mds-chevron-down text-1" })), h("mx-menu", { ref: el => (this.rowsMenu = el), onMxClose: e => e.stopPropagation() }, this.rowsPerPageOptions.map(option => (h("mx-menu-item", { disabled: this.disabled, onClick: this.onChangeRowsPerPage.bind(this, option) }, option))))))));
  }
  static get is() { return "mx-pagination"; }
  static get properties() {
    return {
      "page": {
        "type": "number",
        "mutable": false,
        "complexType": {
          "original": "number",
          "resolved": "number",
          "references": {}
        },
        "required": false,
        "optional": false,
        "docs": {
          "tags": [],
          "text": ""
        },
        "attribute": "page",
        "reflect": false,
        "defaultValue": "1"
      },
      "rowsPerPageOptions": {
        "type": "unknown",
        "mutable": false,
        "complexType": {
          "original": "number[]",
          "resolved": "number[]",
          "references": {}
        },
        "required": false,
        "optional": false,
        "docs": {
          "tags": [],
          "text": ""
        },
        "defaultValue": "[10, 25, 50, 100]"
      },
      "rowsPerPage": {
        "type": "number",
        "mutable": false,
        "complexType": {
          "original": "number",
          "resolved": "number",
          "references": {}
        },
        "required": false,
        "optional": false,
        "docs": {
          "tags": [],
          "text": ""
        },
        "attribute": "rows-per-page",
        "reflect": false,
        "defaultValue": "100"
      },
      "totalRows": {
        "type": "number",
        "mutable": false,
        "complexType": {
          "original": "number",
          "resolved": "number",
          "references": {}
        },
        "required": false,
        "optional": false,
        "docs": {
          "tags": [],
          "text": "Total number of results. If not provided, the page number and last-page buttons will be hidden"
        },
        "attribute": "total-rows",
        "reflect": false
      },
      "disabled": {
        "type": "boolean",
        "mutable": false,
        "complexType": {
          "original": "boolean",
          "resolved": "boolean",
          "references": {}
        },
        "required": false,
        "optional": false,
        "docs": {
          "tags": [],
          "text": "Disable the page buttons (i.e. when loading results)"
        },
        "attribute": "disabled",
        "reflect": false,
        "defaultValue": "false"
      },
      "disableNextPage": {
        "type": "boolean",
        "mutable": false,
        "complexType": {
          "original": "boolean",
          "resolved": "boolean",
          "references": {}
        },
        "required": false,
        "optional": false,
        "docs": {
          "tags": [],
          "text": "Disable the next page button (i.e. when the last page was loaded from an API)"
        },
        "attribute": "disable-next-page",
        "reflect": false,
        "defaultValue": "false"
      }
    };
  }
  static get states() {
    return {
      "hideRangeAndRowsPerPage": {}
    };
  }
  static get events() {
    return [{
        "method": "mxPageChange",
        "name": "mxPageChange",
        "bubbles": true,
        "cancelable": true,
        "composed": true,
        "docs": {
          "tags": [],
          "text": "Emitted when the page or rows-per-page changes"
        },
        "complexType": {
          "original": "PageChangeEventDetail",
          "resolved": "{ rowsPerPage: number; page: number; }",
          "references": {
            "PageChangeEventDetail": {
              "location": "local",
              "path": "/app/packages/mds/src/components/mx-pagination/mx-pagination.tsx",
              "id": "src/components/mx-pagination/mx-pagination.tsx::PageChangeEventDetail"
            }
          }
        }
      }];
  }
  static get elementRef() { return "element"; }
  static get watchers() {
    return [{
        "propName": "totalRows",
        "methodName": "setBreakpoint"
      }, {
        "propName": "rowsPerPage",
        "methodName": "setBreakpoint"
      }, {
        "propName": "rowsPerPageOptions",
        "methodName": "setBreakpoint"
      }];
  }
}
//# sourceMappingURL=mx-pagination.js.map
