import "../../../utils/resizeObserver.mock";
import "../../../utils/matchMedia.mock";
import { newSpecPage } from "@stencil/core/testing";
import { MxPagination } from "../mx-pagination";
import { MxIconButton } from "../../mx-icon-button/mx-icon-button";
import { MxMenu } from "../../mx-menu/mx-menu";
import { MxMenuItem } from "../../mx-menu-item/mx-menu-item";
describe('mx-pagination', () => {
  let page;
  let root;
  let firstPageButton;
  let prevPageButton;
  let nextPageButton;
  let lastPageButton;
  let pageNumbers;
  beforeEach(async () => {
    page = await newSpecPage({
      components: [MxPagination, MxIconButton, MxMenu, MxMenuItem],
      html: `<mx-pagination
        page="3"
        rows-per-page="25"
        total-rows="100"
      >
        <div slot="status">Status</div>
      </mx-pagination>`,
    });
    root = page.root;
    firstPageButton = root.querySelector('[aria-label="First page"]');
    prevPageButton = root.querySelector('[aria-label="Previous page"]');
    nextPageButton = root.querySelector('[aria-label="Next page"]');
    lastPageButton = root.querySelector('[aria-label="Last page"]');
    pageNumbers = root.querySelector('[data-testid="page-numbers"]');
  });
  it('sets the rowsPerPage based on the prop', () => {
    const rowsPerPage = root.querySelector('[data-testid="rows-per-page"]');
    expect(rowsPerPage.innerText).toContain('25');
  });
  it('displays the row range', () => {
    const rowRange = root.querySelector('[data-testid="row-range"]');
    expect(rowRange.innerText).toBe('Displaying 51–75 of 100');
  });
  it('displays up to five page numbers', async () => {
    expect(pageNumbers.innerText).toBe('1234');
    root.rowsPerPage = 10;
    await page.waitForChanges();
    expect(pageNumbers.innerText).toBe('12345');
    root.page = 6;
    await page.waitForChanges();
    expect(pageNumbers.innerText).toBe('678910');
    root.totalRows = 200;
    root.page = 9;
    await page.waitForChanges();
    expect(pageNumbers.innerText).toBe('7891011');
  });
  it('sets the aria-current attribute to the current page', () => {
    expect(root.querySelector('[aria-label="Page 3"]').getAttribute('aria-current')).toBe('page');
  });
  it('disables the first-page and previous-page buttons for the first page', async () => {
    expect(firstPageButton.getAttribute('disabled')).toBeNull();
    expect(prevPageButton.getAttribute('disabled')).toBeNull();
    root.page = 1;
    await page.waitForChanges();
    expect(firstPageButton.getAttribute('disabled')).not.toBeNull();
    expect(prevPageButton.getAttribute('disabled')).not.toBeNull();
  });
  it('disables the last-page and next-page buttons for the last page', async () => {
    expect(lastPageButton.getAttribute('disabled')).toBeNull();
    expect(nextPageButton.getAttribute('disabled')).toBeNull();
    root.page = 4;
    await page.waitForChanges();
    expect(lastPageButton.getAttribute('disabled')).not.toBeNull();
    expect(nextPageButton.getAttribute('disabled')).not.toBeNull();
  });
  it('disables the buttons when the disabled prop is set', async () => {
    root.disabled = true;
    await page.waitForChanges();
    expect(firstPageButton.getAttribute('disabled')).not.toBeNull();
    expect(prevPageButton.getAttribute('disabled')).not.toBeNull();
    expect(lastPageButton.getAttribute('disabled')).not.toBeNull();
    expect(nextPageButton.getAttribute('disabled')).not.toBeNull();
  });
  it('disables the next-page button when the disableNextPage prop is set', async () => {
    root.disableNextPage = true;
    await page.waitForChanges();
    expect(lastPageButton.getAttribute('disabled')).toBeNull();
    expect(nextPageButton.getAttribute('disabled')).not.toBeNull();
  });
  it('sets the rows-per-page options based on the rowsPerPageOptions prop', async () => {
    root.rowsPerPageOptions = [3, 9];
    root.rowsPerPage = 3;
    await page.waitForChanges();
    const rowsPerPageMenuItems = root.querySelectorAll('mx-menu-item');
    expect(rowsPerPageMenuItems.length).toBe(2);
    expect(rowsPerPageMenuItems[0].innerText).toBe('3');
    expect(rowsPerPageMenuItems[1].innerText).toBe('9');
  });
  it('emits an mxPageChange event when the rows-per-page is changed', async () => {
    let emittedValue;
    root.addEventListener('mxPageChange', (e) => (emittedValue = e.detail));
    const firstMenuItem = root.querySelector('mx-menu-item');
    firstMenuItem.click();
    expect(emittedValue.page).toBe(1);
    expect(emittedValue.rowsPerPage).toBe(10);
  });
  it('emits an mxPageChange event when the page is changed', async () => {
    let emittedValue;
    root.addEventListener('mxPageChange', (e) => (emittedValue = e.detail));
    nextPageButton.click();
    expect(emittedValue.page).toBe(4);
    expect(emittedValue.rowsPerPage).toBe(25);
  });
});
//# sourceMappingURL=mx-pagination.spec.js.map
