import { Host, h } from "@stencil/core";
import { queryPrefersReducedMotion } from "../../utils/utils";
import { MinWidths, minWidthSync } from "../../utils/minWidthSync";
export class MxTabs {
  constructor() {
    this.tabEls = [];
    this.menuItemEls = [];
    this.hasMenuSlot = false;
    this.tabBreakpoints = new Map();
    this.tabVisibility = new Map();
    this.value = null;
    this.tabs = undefined;
    this.menuButton = undefined;
    this.nav = false;
    this.submenu = false;
    this.minWidths = new MinWidths();
    this.showMoreTab = true;
    this.isMoreSelected = false;
  }
  onMenuOpen(e) {
    if (e.target === this.menuEl)
      this.mxMenuOpen.emit();
  }
  onMenuClose(e) {
    if (e.target === this.menuEl)
      this.mxMenuClose.emit();
  }
  connectedCallback() {
    minWidthSync.subscribeComponent(this);
  }
  resetTabVisibility() {
    if ((!this.nav && !this.submenu) || !this.tabEls.length)
      return;
    this.setTabBreakpoints();
    this.updateTabVisibility();
  }
  onValueChange(tabIndex, previousTabIndex) {
    const isTabHidden = index => this.nav && !this.tabVisibility.get(this.tabEls[index]);
    const moreTabElIndex = this.tabEls.length;
    if (isTabHidden(tabIndex))
      tabIndex = moreTabElIndex;
    if (isTabHidden(previousTabIndex))
      previousTabIndex = moreTabElIndex;
    this.animateIndicator(tabIndex, previousTabIndex);
    requestAnimationFrame(() => (this.isMoreSelected = isTabHidden(tabIndex)));
  }
  animateIndicator(tabIndex, previousTabIndex) {
    if (queryPrefersReducedMotion())
      return;
    if (tabIndex == null || previousTabIndex == null)
      return;
    const tabEls = this.element.querySelectorAll('.mx-tab');
    const previousSelectedTab = tabEls[previousTabIndex];
    const newSelectedTab = tabEls[tabIndex];
    if (!previousSelectedTab || !newSelectedTab)
      return;
    const distance = previousSelectedTab.offsetLeft - newSelectedTab.offsetLeft;
    const scaleX = previousSelectedTab.offsetWidth / newSelectedTab.offsetWidth;
    const indicator = newSelectedTab.querySelector('.active-tab-indicator');
    if (!indicator)
      return;
    indicator.style.transform = `translateX(${distance}px) scale3d(${scaleX}, 1, 1)`;
    indicator.style.transformOrigin = 'left';
    indicator.style.transition = `none`;
    setTimeout(() => {
      indicator.style.transform = `translateX(0) scale3d(1, 1, 1)`;
      indicator.style.transition = `transform 0.25s cubic-bezier(0.4, 0, 0.2, 1)`;
    }, 0);
  }
  componentWillRender() {
    this.hasMenuSlot = !!this.element.querySelector('[slot="menu"]');
  }
  componentDidLoad() {
    if (this.nav || this.submenu) {
      this.resetTabVisibility();
      const resizeObserver = new ResizeObserver(() => {
        if (this.menuEl)
          this.menuEl.closeMenu();
        this.updateTabVisibility();
      });
      resizeObserver.observe(this.element);
    }
  }
  disconnectedCallback() {
    minWidthSync.unsubscribeComponent(this);
  }
  setTabBreakpoints() {
    this.showAllTabs();
    this.tabBreakpoints.clear();
    const { left: containerLeft } = this.element.getBoundingClientRect();
    const lastTabRight = this.tabEls[this.tabEls.length - 1].getBoundingClientRect().right;
    const moreTabOffset = this.moreTabEl && this.moreTabEl.getBoundingClientRect().right - lastTabRight;
    this.tabEls.forEach((tab, i) => {
      let breakpoint;
      if (this.submenu) {
        breakpoint = lastTabRight - containerLeft;
      }
      else {
        const { right } = tab.getBoundingClientRect();
        breakpoint = right - containerLeft;
      }
      if (this.moreTabEl) {
        if (i < this.tabs.length - 1)
          breakpoint += moreTabOffset;
        if (i === this.tabs.length - 1) {
          const previousTab = this.tabEls[i - 1];
          const previousTabBreakpoint = this.tabBreakpoints.get(previousTab);
          if (previousTabBreakpoint > breakpoint)
            this.tabBreakpoints.set(previousTab, breakpoint);
        }
      }
      this.tabBreakpoints.set(tab, breakpoint);
    });
    if (this.submenu)
      return;
    if (this.tabs.length === 3)
      this.tabBreakpoints.set(this.tabEls[1], this.tabBreakpoints.get(this.tabEls[2]));
    if (this.tabs.length > 1)
      this.tabBreakpoints.set(this.tabEls[0], this.tabBreakpoints.get(this.tabEls[1]));
  }
  showAllTabs() {
    this.tabEls.forEach(tab => {
      this.tabVisibility.set(tab, true);
    });
    this.applyTabVisibility();
  }
  updateTabVisibility() {
    let { width } = this.element.getBoundingClientRect();
    if (this.allTabsHidden && this.menuButton)
      width += this.menuButton.scrollWidth;
    let isAnyTabHidden = false;
    let isHiddenTabSelected = false;
    let isChanging = false;
    this.tabBreakpoints.forEach((breakpoint, tab) => {
      const isVisible = width >= breakpoint && (this.submenu || this.minWidths.sm);
      isChanging = isChanging || isVisible !== this.tabVisibility.get(tab);
      this.tabVisibility.set(tab, isVisible);
      if (!isVisible) {
        isAnyTabHidden = true;
        if (tab.selected)
          isHiddenTabSelected = true;
      }
    });
    if (this.moreTabEl)
      this.updateMoreTab(isAnyTabHidden, isHiddenTabSelected);
    if (isChanging)
      this.applyTabVisibility();
    if (this.menuButton)
      this.updateMenuButton();
  }
  updateMoreTab(isAnyTabHidden = false, isHiddenTabSelected = false) {
    requestAnimationFrame(() => {
      const canShowMoreTab = !this.allTabsHidden;
      this.showMoreTab = canShowMoreTab && isAnyTabHidden;
      this.isMoreSelected = isHiddenTabSelected;
      if (this.showMoreTab) {
        this.menuEl.anchorEl = this.moreTabEl.firstElementChild;
      }
    });
  }
  updateMenuButton() {
    this.menuButton.classList.toggle('hidden', !this.allTabsHidden);
    if (this.submenu)
      this.submenuLabel.classList.toggle('hidden', !this.allTabsHidden);
    if (this.allTabsHidden)
      this.menuEl.anchorEl = this.menuButton;
  }
  applyTabVisibility() {
    if (this.menuEl)
      this.menuEl.closeMenu();
    this.tabEls.forEach((tab, index) => {
      if (this.tabVisibility.get(tab)) {
        tab.classList.remove('hidden');
        if (this.menuItemEls[index])
          this.menuItemEls[index].classList.add('hidden');
      }
      else {
        tab.classList.add('hidden');
        if (this.menuItemEls[index])
          this.menuItemEls[index].classList.remove('hidden');
      }
    });
  }
  onClick(e) {
    let tab = e.target.closest('.mx-tab');
    const menuItem = e.target.closest('mx-menu-item');
    if (menuItem)
      tab = this.tabEls[+menuItem.dataset.tabIndex];
    if (!tab)
      return;
    const tabIndex = Array.prototype.indexOf.call(this.tabEls, tab);
    if (tabIndex >= 0)
      this.mxChange.emit(tabIndex);
    if (menuItem && tab.href)
      tab.querySelector('a').click();
  }
  onInput(e) {
    this.mxChange.emit(+e.target.value);
  }
  get allTabsHidden() {
    return !this.tabVisibility.get(this.tabEls[0]);
  }
  get selectedTabLabel() {
    return this.tabs[this.value].label || this.tabs[this.value].elAriaLabel;
  }
  get menuEl() {
    if (this.allTabsHidden && this.hasMenuSlot) {
      return this.element.querySelector('mx-menu[slot="menu"]');
    }
    return this.moreMenuEl;
  }
  get renderAsSelect() {
    if (this.nav || this.submenu || this.minWidths.md)
      return false;
    return this.tabs && this.tabs.length > 2;
  }
  get gridClass() {
    let str = 'inline-grid auto-cols-min grid-flow-col';
    str += this.nav ? ' gap-x-24' : ' gap-x-20';
    return str;
  }
  render() {
    return (h(Host, { class: "mx-tabs relative block", role: "tablist" }, this.renderAsSelect ? (h("mx-select", { value: this.value, onInput: this.onInput.bind(this), dense: true }, this.tabs.map((tab, index) => (h("option", { value: index }, tab.label || tab.elAriaLabel))))) : (this.tabs && (h("div", { class: this.gridClass }, this.tabs.map((tab, index) => (h("mx-tab", Object.assign({ ref: el => (this.tabEls[index] = el), key: 'tab-' + index, selected: this.value === index, nav: this.nav }, tab)))), this.nav && this.tabs.length > 3 && (h("mx-tab", { ref: el => (this.moreTabEl = el), class: this.showMoreTab ? '' : 'invisible', label: "More", "el-role": "button", icon: "mds-arrow-triangle-down", selected: this.isMoreSelected, nav: true, "data-testid": "more-tab" })), this.submenu && (h("div", { ref: el => (this.submenuLabel = el), class: "submenu-label h-60 text-small hidden", "data-testid": "submenu-label" }, h("span", { class: "flex h-full items-center uppercase tracking-0-3 font-bold whitespace-nowrap" }, this.selectedTabLabel)))))), h("slot", { name: "menu" }), (this.nav || this.submenu) && (h("mx-menu", { ref: el => (this.moreMenuEl = el), squared: true }, this.tabs.map((tab, index) => (h("mx-menu-item", { ref: el => (this.menuItemEls[index] = el), key: 'menu-item-' + index, selected: this.value === index || tab.selected, "data-tab-index": index }, tab.label || tab.elAriaLabel)))))));
  }
  static get is() { return "mx-tabs"; }
  static get properties() {
    return {
      "value": {
        "type": "number",
        "mutable": false,
        "complexType": {
          "original": "number",
          "resolved": "number",
          "references": {}
        },
        "required": false,
        "optional": false,
        "docs": {
          "tags": [],
          "text": "The index of the selected tab"
        },
        "attribute": "value",
        "reflect": false,
        "defaultValue": "null"
      },
      "tabs": {
        "type": "unknown",
        "mutable": false,
        "complexType": {
          "original": "IMxTabProps[]",
          "resolved": "IMxTabProps[]",
          "references": {
            "IMxTabProps": {
              "location": "import",
              "path": "../mx-tab/mx-tab",
              "id": "src/components/mx-tab/mx-tab.tsx::IMxTabProps"
            }
          }
        },
        "required": true,
        "optional": false,
        "docs": {
          "tags": [],
          "text": "An array of objects for each tab (see [IMxTabProps Properties](#properties))"
        }
      },
      "menuButton": {
        "type": "unknown",
        "mutable": false,
        "complexType": {
          "original": "HTMLElement",
          "resolved": "HTMLElement",
          "references": {
            "HTMLElement": {
              "location": "global",
              "id": "global::HTMLElement"
            }
          }
        },
        "required": false,
        "optional": false,
        "docs": {
          "tags": [],
          "text": "The hamburger menu button from a parent nav component"
        }
      },
      "nav": {
        "type": "boolean",
        "mutable": false,
        "complexType": {
          "original": "boolean",
          "resolved": "boolean",
          "references": {}
        },
        "required": false,
        "optional": false,
        "docs": {
          "tags": [],
          "text": "When set to true, enables the \"More\" menu and changes the styling for use inside the [Top Navigation](/components/navigation.html#top-navigation) component"
        },
        "attribute": "nav",
        "reflect": true,
        "defaultValue": "false"
      },
      "submenu": {
        "type": "boolean",
        "mutable": false,
        "complexType": {
          "original": "boolean",
          "resolved": "boolean",
          "references": {}
        },
        "required": false,
        "optional": false,
        "docs": {
          "tags": [],
          "text": "Used internally for the mx-submenu component. When set to true, the tabs overflow into a menu when any tab does not fit"
        },
        "attribute": "submenu",
        "reflect": true,
        "defaultValue": "false"
      }
    };
  }
  static get states() {
    return {
      "minWidths": {},
      "showMoreTab": {},
      "isMoreSelected": {}
    };
  }
  static get events() {
    return [{
        "method": "mxChange",
        "name": "mxChange",
        "bubbles": true,
        "cancelable": true,
        "composed": true,
        "docs": {
          "tags": [],
          "text": "Emits the newly selected tab's index as `Event.detail`"
        },
        "complexType": {
          "original": "number",
          "resolved": "number",
          "references": {}
        }
      }, {
        "method": "mxMenuOpen",
        "name": "mxMenuOpen",
        "bubbles": true,
        "cancelable": true,
        "composed": true,
        "docs": {
          "tags": [],
          "text": "Emitted when the menu is opened"
        },
        "complexType": {
          "original": "void",
          "resolved": "void",
          "references": {}
        }
      }, {
        "method": "mxMenuClose",
        "name": "mxMenuClose",
        "bubbles": true,
        "cancelable": true,
        "composed": true,
        "docs": {
          "tags": [],
          "text": "Emitted when the menu is closed"
        },
        "complexType": {
          "original": "void",
          "resolved": "void",
          "references": {}
        }
      }];
  }
  static get elementRef() { return "element"; }
  static get watchers() {
    return [{
        "propName": "tabs",
        "methodName": "resetTabVisibility"
      }, {
        "propName": "minWidths",
        "methodName": "resetTabVisibility"
      }, {
        "propName": "menuButton",
        "methodName": "resetTabVisibility"
      }, {
        "propName": "value",
        "methodName": "onValueChange"
      }];
  }
  static get listeners() {
    return [{
        "name": "mxOpen",
        "method": "onMenuOpen",
        "target": undefined,
        "capture": false,
        "passive": false
      }, {
        "name": "mxClose",
        "method": "onMenuClose",
        "target": undefined,
        "capture": false,
        "passive": false
      }, {
        "name": "click",
        "method": "onClick",
        "target": undefined,
        "capture": false,
        "passive": false
      }];
  }
}
//# sourceMappingURL=mx-tabs.js.map
