import { newSpecPage } from "@stencil/core/testing";
import { MxTopNav } from "../mx-top-nav";
import { MxTabs } from "../../mx-tabs/mx-tabs";
import { MxTab } from "../../mx-tab/mx-tab";
import { MxMenu } from "../../mx-menu/mx-menu";
import "../../../utils/resizeObserver.mock";
describe('mx-top-nav', () => {
  let page;
  let root;
  beforeEach(async () => {
    page = await newSpecPage({
      components: [MxTopNav, MxTabs, MxTab, MxMenu],
      html: `
      <mx-top-nav>
        <h1 slot="header">MoxiWorks</h1>
        <img src="blah.png" slot="avatar">
      </mx-top-nav>`,
    });
    await page.waitForChanges();
    await page.waitForChanges();
    root = page.root;
  });
  it('renders the header slot correctly', async () => {
    const headerContent = root.querySelector('header h1[slot="header"]');
    expect(headerContent).not.toBeNull();
  });
  it('renders the avatar slot correctly', async () => {
    const avatarContent = root.querySelector('[data-testid="avatar"] img[slot="avatar"]');
    expect(avatarContent).not.toBeNull();
  });
  it('renders search when showSearch is true', async () => {
    let searchInput = root.querySelector('input[type="search"]');
    expect(searchInput).not.toBeNull();
    root.showSearch = false;
    await page.waitForChanges();
    searchInput = root.querySelector('input[type="search"]');
    expect(searchInput).toBeNull();
  });
  it('sets the search input attributes', async () => {
    root.searchForm = 'my-form';
    root.searchId = 'my-id';
    root.searchName = 'my-name';
    root.searchValue = 'my-value';
    root.searchPlaceholder = 'my-placeholder';
    await page.waitForChanges();
    const searchInput = root.querySelector('input[type="search"]');
    expect(searchInput.form).toBe('my-form');
    expect(searchInput.id).toBe('my-id');
    expect(searchInput.name).toBe('my-name');
    expect(searchInput.value).toBe('my-value');
    expect(searchInput.placeholder).toBe('my-placeholder');
  });
  it('shows search spinner when searchLoading is true', async () => {
    const searchInput = root.querySelector('input[type="search"]');
    root.searchLoading = true;
    searchInput.focus();
    await page.waitForChanges();
    const spinner = root.querySelector('[data-testid="search-spinner"]');
    expect(spinner).not.toBeNull();
  });
  it('emits mxSearch event on pressing Enter in search input', async () => {
    const searchInput = root.querySelector('input[type="search"]');
    searchInput.value = 'hello';
    const mockEvent = new KeyboardEvent('keydown', { key: 'Enter' });
    let emittedValue;
    root.addEventListener('mxSearch', (e) => (emittedValue = e.detail));
    searchInput.dispatchEvent(mockEvent);
    expect(emittedValue).toBe('hello');
  });
  it('renders tabs using the mx-tabs component when provided tabs', async () => {
    let tabsComponent = root.querySelector('mx-tabs');
    expect(tabsComponent).toBeNull();
    root.tabs = [{ label: 'Home' }, { label: 'Dashboard' }];
    await page.waitForChanges();
    tabsComponent = root.querySelector('mx-tabs');
    expect(tabsComponent).not.toBeNull();
  });
  it('emits mxChange event when a tab is selected', async () => {
    let emittedValue;
    root.tabs = [{ label: 'Home' }, { label: 'Dashboard' }];
    root.selectedTabIndex = 1;
    await page.waitForChanges();
    expect(root.querySelectorAll('mx-tab')[1].selected).toBe(true);
    root.addEventListener('mxChange', (e) => (emittedValue = e.detail));
    const tab = root.querySelector('mx-tab');
    tab.click();
    expect(emittedValue).toBe(0);
  });
});
describe('mx-top-nav (menu slot)', () => {
  let page;
  let root;
  beforeEach(async () => {
    page = await newSpecPage({
      components: [MxTopNav, MxTabs, MxTab, MxMenu],
      html: `
      <mx-top-nav>
        <mx-menu slot="menu">
          <p role="heading">My Menu</p>
          <mx-menu-item>Home</mx-menu-item>
          <mx-menu-item>Dashboard</mx-menu-item>
          <mx-menu-item>Log Out</mx-menu-item>
        </mx-menu>
      </mx-top-nav>`,
    });
    await page.waitForChanges();
    root = page.root;
    root.tabs = [{ label: 'Home' }, { label: 'Dashboard' }];
    await page.waitForChanges();
  });
  it('renders the menu slot inside the inner mx-tabs element', async () => {
    expect(root.querySelector('mx-tabs mx-menu[slot="menu"]')).not.toBeNull();
  });
});
//# sourceMappingURL=mx-top-nav.spec.js.map
