import { proxyCustomElement, HTMLElement, createEvent, h, Host } from '@stencil/core/internal/client';
import { M as MinWidths, m as minWidthSync } from './minWidthSync.js';
import { d as defineCustomElement$1 } from './mx-checkbox2.js';

const MxMenuItem = proxyCustomElement(class MxMenuItem extends HTMLElement {
  constructor() {
    super();
    this.__registerHost();
    this.mxClick = createEvent(this, "mxClick", 7);
    this.hasParentLink = false;
    this.checked = false;
    this.disabled = false;
    this.icon = undefined;
    this.label = undefined;
    this.subtitle = undefined;
    this.multiSelect = false;
    this.selected = false;
    this.minWidths = new MinWidths();
    this.accordion = undefined;
  }
  onMouseEnter() {
    this.closeSiblingSubMenus();
    this.focusMenuItem();
    if (this.submenu) {
      clearTimeout(this.submenuDelayTimeout);
      this.submenuDelayTimeout = setTimeout(this.openSubMenu.bind(this), 150);
    }
  }
  onMouseLeave() {
    clearTimeout(this.submenuDelayTimeout);
    if (!this.disabled)
      document.activeElement.blur();
  }
  onFocus() {
    this.closeSiblingSubMenus();
  }
  onKeyDown(e) {
    if (this.accordion)
      return this.onKeyDownAccordion(e);
    if (this.submenu)
      return this.onKeyDownSubMenu(e);
    const clickKeys = ['Enter'];
    if (this.multiSelect)
      clickKeys.push(' ');
    if (clickKeys.includes(e.key)) {
      e.preventDefault();
      e.stopPropagation();
      document.activeElement.click();
    }
  }
  componentWillLoad() {
    this.submenu = this.element.querySelector('[slot="submenu"]');
    this.accordion = this.element.querySelector('[slot="accordion"]');
  }
  componentDidLoad() {
    if (this.accordion)
      this.accordion.anchorEl = this.menuItemElem;
  }
  connectedCallback() {
    const parentLink = this.element.closest('a');
    if (parentLink) {
      this.hasParentLink = true;
      parentLink.setAttribute('role', 'menuitem');
    }
    else {
      this.role = this.element.closest('mx-dropdown-menu') ? 'option' : 'menuitem';
    }
    minWidthSync.subscribeComponent(this);
  }
  disconnectedCallback() {
    minWidthSync.unsubscribeComponent(this);
  }
  async closeAccordion(skipTransition = false) {
    if (this.accordion) {
      return await this.accordion.closeMenu(skipTransition);
    }
  }
  async closeSubMenu() {
    if (this.submenu) {
      clearTimeout(this.submenuDelayTimeout);
      return await this.submenu.closeMenu();
    }
  }
  async getValue() {
    return this.slotWrapper && this.slotWrapper.innerText.trim();
  }
  async focusMenuItem() {
    if (this.disabled)
      return;
    if (this.multiSelect) {
      const label = this.menuItemElem.querySelector('mx-checkbox label');
      label && label.focus();
    }
    else {
      this.menuItemElem.focus();
    }
  }
  onKeyDownAccordion(e) {
    if (!['Enter', ' ', 'ArrowRight', 'ArrowLeft'].includes(e.key))
      return;
    e.preventDefault();
    e.stopPropagation();
    if (e.key === 'ArrowRight') {
      this.accordion.openMenu();
    }
    else if (e.key === 'ArrowLeft') {
      this.accordion.closeMenu();
      this.focusMenuItem();
    }
    else {
      document.activeElement.click();
    }
  }
  async onKeyDownSubMenu(e) {
    if (['Enter', ' ', 'ArrowRight'].includes(e.key)) {
      e.preventDefault();
      e.stopPropagation();
      const didOpen = await this.openSubMenu();
      if (didOpen) {
        const firstMenuItem = this.element.querySelector('mx-menu-item:not(:disabled)');
        firstMenuItem && firstMenuItem.focusMenuItem();
      }
    }
    else if (e.key === 'ArrowLeft') {
      e.preventDefault();
      e.stopPropagation();
      const didClose = await this.closeSubMenu();
      if (didClose) {
        this.focusMenuItem();
      }
      else {
        this.element.parentElement.dispatchEvent(new KeyboardEvent(e.type, e));
      }
    }
  }
  closeSiblingSubMenus() {
    const siblingMenuItems = Array.from(this.element.parentElement.children).filter(e => e !== this.element && e.tagName === 'MX-MENU-ITEM');
    siblingMenuItems.forEach((m) => m.closeSubMenu());
  }
  openSubMenu() {
    if (this.submenu) {
      this.submenu.placement = 'right-start';
      this.submenu.anchorEl = this.menuItemElem;
      return this.submenu.openMenu();
    }
  }
  onClick(e) {
    if (this.disabled || !!this.submenu || !!this.accordion) {
      e.stopPropagation();
      e.preventDefault();
      return;
    }
    if (!this.multiSelect)
      this.mxClick.emit(e);
  }
  get checkboxLabel() {
    return (this.slotWrapper || this.element).innerText;
  }
  render() {
    return (h(Host, { role: "none", class: 'mx-menu-item block' + (this.submenu ? ' has-submenu' : '') }, h("div", { ref: el => (this.menuItemElem = el), role: this.role, "aria-checked": this.role === 'option' ? (this.checked ? 'true' : 'false') : null, "aria-disabled": this.disabled ? 'true' : null, "aria-haspopup": this.submenu ? 'true' : null, "aria-selected": this.selected ? 'true' : null, tabindex: this.disabled || this.multiSelect || this.hasParentLink ? '-1' : '0', class: "block w-full cursor-pointer select-none text-4 outline-none rounded disabled:cursor-auto", onClick: this.onClick.bind(this) }, this.label && (h("p", { class: "item-label flex items-end py-0 pl-14 pr-10 my-0 h-18 uppercase subtitle5" }, h("span", { class: "block -mb-4" }, this.label))), h("div", { class: 'flex items-center w-full justify-between pl-14 pr-10 h-40 whitespace-nowrap' +
        (this.multiSelect ? ' hidden' : '') }, h("div", { class: "flex items-center w-full h-full" }, this.icon !== undefined && (h("i", { class: 'inline-flex items-center justify-center text-1 w-20 mr-8 ' + this.icon })), h("span", { ref: el => (this.slotWrapper = el), class: "truncate" }, h("slot", null)), h("span", { class: "ml-auto" }, h("slot", { name: "right" }))), this.checked && !this.multiSelect && h("i", { class: "check mds-check text-icon ml-14", "data-testid": "check" }), !!this.accordion && (h("i", { class: "accordion-arrow mds-chevron-right text-icon transform transition-transform", "data-testid": "accordion-arrow" })), !!this.submenu && (h("i", { class: "mds-arrow-triangle-down text-icon transform -rotate-90", "data-testid": "arrow" }))), this.subtitle && (h("p", { class: "item-subtitle flex items-start py-0 pl-14 pr-10 my-0 h-16 caption2" }, h("span", { class: "block -mt-4 truncate" }, this.subtitle))), this.multiSelect && (h("mx-checkbox", { class: "flex items-stretch w-full overflow-hidden h-48 sm:h-32", "label-class": "pl-14 pr-16", checked: this.checked, "label-name": this.checkboxLabel, "label-left": !this.minWidths.sm }))), h("slot", { name: "accordion" }), h("slot", { name: "submenu" })));
  }
  get element() { return this; }
}, [4, "mx-menu-item", {
    "checked": [4],
    "disabled": [4],
    "icon": [1],
    "label": [1],
    "subtitle": [1],
    "multiSelect": [4, "multi-select"],
    "selected": [4],
    "minWidths": [32],
    "accordion": [32],
    "closeAccordion": [64],
    "closeSubMenu": [64],
    "getValue": [64],
    "focusMenuItem": [64]
  }, [[1, "mouseenter", "onMouseEnter"], [1, "mouseleave", "onMouseLeave"], [0, "focus", "onFocus"], [0, "keydown", "onKeyDown"]]]);
function defineCustomElement() {
  if (typeof customElements === "undefined") {
    return;
  }
  const components = ["mx-menu-item", "mx-checkbox"];
  components.forEach(tagName => { switch (tagName) {
    case "mx-menu-item":
      if (!customElements.get(tagName)) {
        customElements.define(tagName, MxMenuItem);
      }
      break;
    case "mx-checkbox":
      if (!customElements.get(tagName)) {
        defineCustomElement$1();
      }
      break;
  } });
}

export { MxMenuItem as M, defineCustomElement as d };

//# sourceMappingURL=mx-menu-item2.js.map