import { proxyCustomElement, HTMLElement, createEvent, h, Host } from '@stencil/core/internal/client';
import { c as createPopover, a as convertPlacementToOrigin } from './popover.js';
import { f as fadeScaleIn, a as fadeOut } from './transitions.js';
import { u as uuidv4 } from './utils.js';

const MxMenu = proxyCustomElement(class MxMenu extends HTMLElement {
  constructor() {
    super();
    this.__registerHost();
    this.mxClose = createEvent(this, "mxClose", 7);
    this.mxOpen = createEvent(this, "mxOpen", 7);
    this.isClosing = false;
    this.uuid = uuidv4();
    this.anchorEl = undefined;
    this.autocompleteOnly = false;
    this.triggerEl = undefined;
    this.offset = undefined;
    this.placement = 'bottom-start';
    this.isOpen = false;
    this.squared = false;
  }
  onMenuItemClick() {
    this.closeMenu();
  }
  onClick(e) {
    const triggerEl = this.triggerEl || this.anchorEl;
    const triggerElWasClicked = triggerEl && triggerEl.contains(e.target);
    if (triggerElWasClicked)
      e.preventDefault();
    if (!this.isOpen && triggerElWasClicked) {
      this.openMenu();
      e.preventDefault();
    }
    else if (this.isOpen && this.element && !this.element.contains(e.target)) {
      if (this.isSubMenu && triggerElWasClicked)
        return;
      if (this.inputEl && this.inputEl.contains(e.target))
        return;
      this.closeMenu();
    }
  }
  onFocus(e) {
    if (!this.anchorEl)
      return;
    if (this.isOpen && !this.anchorEl.contains(e.target) && !this.element.contains(e.target))
      this.closeMenu();
  }
  onDocumentKeyDown(e) {
    const isFocused = (el) => el && el.contains(e.target);
    const enabledMenuItems = this.menuItems.filter(m => !m.disabled);
    if (this.autocompleteOnly && this.inputEl && this.isOpen && e.key === 'Enter' && !isFocused(this.element)) {
      e.preventDefault();
      enabledMenuItems.length && enabledMenuItems[0].click();
      this.onMenuItemClick();
      return;
    }
    const openKeys = ['Enter'];
    if (!this.inputEl)
      openKeys.push(' ');
    if (openKeys.includes(e.key) && this.anchorEl && isFocused(this.anchorEl)) {
      (this.triggerEl || this.anchorEl).click();
      e.preventDefault();
      e.stopPropagation();
      return;
    }
    if (!this.isOpen && this.inputEl && isFocused(this.inputEl)) {
      if (['Escape', 'Tab'].includes(e.key))
        return;
      (this.triggerEl || this.anchorEl).click();
    }
    if (!this.isOpen)
      return;
    const shouldEditInput = e.key.length === 1 || ['Backspace', 'Delete', 'ArrowLeft', 'ArrowRight'].includes(e.key);
    if (this.inputEl && shouldEditInput && !isFocused(this.inputEl)) {
      this.inputEl.focus();
    }
    else if (e.key === 'Escape') {
      this.closeMenu();
      e.preventDefault();
    }
    else if (['Tab', 'ArrowDown'].includes(e.key) && isFocused(this.anchorEl) && enabledMenuItems.length > 0) {
      if (e.shiftKey && e.key === 'Tab')
        return;
      if (!this.inputEl || !this.autocompleteOnly) {
        enabledMenuItems[0].focusMenuItem();
      }
      else if (this.autocompleteOnly && enabledMenuItems.length >= 2) {
        enabledMenuItems[1].focusMenuItem();
      }
      e.preventDefault();
      e.stopPropagation();
    }
    else if (e.key === 'ArrowUp' && isFocused(this.inputEl)) {
      e.preventDefault();
    }
    else if (this.inputEl && isFocused(this.inputEl)) {
      this.isOpen = !this.isOpen;
      this.isOpen = !this.isOpen;
    }
  }
  onKeydown(e) {
    if (!this.isOpen)
      return;
    if (!['ArrowDown', 'ArrowUp'].includes(e.key))
      return;
    e.preventDefault();
    e.stopPropagation();
    const tabbableMenuItems = [];
    this.menuItems.forEach(m => {
      if (m.disabled)
        return;
      tabbableMenuItems.push(m);
      const accordionMenuItems = m.querySelectorAll('[aria-expanded="true"] + mx-menu-accordion mx-menu-item');
      tabbableMenuItems.push(...Array.from(accordionMenuItems));
    });
    const focusedMenuItem = document.activeElement ? document.activeElement.closest('mx-menu-item') : null;
    const focusedIndex = tabbableMenuItems.findIndex(m => m === focusedMenuItem);
    if (e.key === 'ArrowDown' && focusedIndex !== tabbableMenuItems.length - 1) {
      tabbableMenuItems[focusedIndex + 1].focusMenuItem();
    }
    else if (e.key === 'ArrowUp' && focusedIndex > 0) {
      tabbableMenuItems[focusedIndex - 1].focusMenuItem();
    }
  }
  async openMenu() {
    if (this.isOpen || !this.anchorEl)
      return false;
    this.isOpen = true;
    this.mxOpen.emit();
    const offset = this.offset || (this.isSubMenu ? [-8, 0] : [0, 1]);
    this.popoverInstance = await createPopover(this.anchorEl, this.element, this.placement, offset);
    await fadeScaleIn(this.menuElem, undefined, convertPlacementToOrigin(this.popoverInstance.state.placement));
    return true;
  }
  setTriggerElAttributes() {
    if (!this.anchorEl && !this.triggerEl)
      return;
    (this.triggerEl || this.anchorEl).setAttribute('aria-haspopup', 'true');
    (this.triggerEl || this.anchorEl).setAttribute('aria-controls', this.uuid);
  }
  async closeMenu() {
    if (!this.isOpen || this.isClosing)
      return false;
    this.isClosing = true;
    this.menuItems.forEach(m => m.closeSubMenu());
    await fadeOut(this.menuElem);
    this.menuItems.forEach(m => m.closeAccordion(true));
    this.isClosing = false;
    this.mxClose.emit();
    this.isOpen = false;
    if (!this.popoverInstance)
      return true;
    this.popoverInstance.destroy();
    this.popoverInstance = null;
    return true;
  }
  componentDidLoad() {
    this.setInputEl();
    if (this.menuItems.length) {
      const role = this.element.querySelector('[role="option"]') ? 'listbox' : 'menu';
      this.scrollElem.setAttribute('role', role);
    }
    this.setTriggerElAttributes();
  }
  componentWillUpdate() {
    this.setInputEl();
    this.anchorEl &&
      this.anchorEl.getAttribute('role') === 'menuitem' &&
      this.anchorEl.setAttribute('aria-expanded', this.isOpen ? 'true' : 'false');
    if (this.inputEl && this.anchorEl)
      this.element.style.width = this.anchorEl.getBoundingClientRect().width + 'px';
    const anyMenuItemHasIcon = this.menuItems.some(m => !!m.icon);
    if (anyMenuItemHasIcon) {
      this.menuItems.forEach(m => {
        if (m.icon === undefined)
          m.icon = null;
      });
    }
    if (this.inputEl) {
      this.menuItems.forEach(async (m) => {
        m.selected = this.inputEl.value === (await m.getValue());
      });
    }
  }
  setInputEl() {
    if (this.anchorEl && !this.inputEl) {
      this.inputEl = this.anchorEl.querySelector('input[type="text"], input[type="search"]');
      this.inputEl && this.inputEl.setAttribute('autocomplete', 'off');
    }
  }
  get menuItems() {
    const menuItems = [];
    Array.from(this.scrollElem.children).forEach((el) => {
      if (el.tagName === 'MX-MENU-ITEM')
        menuItems.push(el);
      if (el.getAttribute('role') === 'group') {
        menuItems.push(...Array.from(el.children).filter(e => e.tagName === 'MX-MENU-ITEM'));
      }
    });
    return menuItems;
  }
  get isSubMenu() {
    return this.element.hasAttribute('slot') && this.element.getAttribute('slot') === 'submenu';
  }
  get hostClass() {
    let str = 'mx-menu block z-50 w-screen sm:w-auto';
    if (!this.isOpen)
      str += ' hidden';
    if (this.autocompleteOnly)
      str += ' autocomplete-only';
    return str;
  }
  get menuClass() {
    let str = 'flex flex-col';
    if (this.squared) {
      str += ' border shadow-drop';
    }
    else {
      str += ' shadow-9 rounded';
    }
    if (!this.isOpen)
      str += ' hidden';
    if (this.autocompleteOnly)
      str += ' autocomplete-only';
    return str;
  }
  render() {
    return (h(Host, { class: this.hostClass }, h("div", { ref: el => (this.menuElem = el), class: this.menuClass }, h("div", { id: this.uuid, ref: el => (this.scrollElem = el), class: "scroll-wrapper overflow-y-auto overflow-x-hidden overscroll-contain" }, h("slot", null)))));
  }
  get element() { return this; }
  static get watchers() { return {
    "anchorEl": ["setTriggerElAttributes"],
    "triggerEl": ["setTriggerElAttributes"]
  }; }
}, [4, "mx-menu", {
    "anchorEl": [16],
    "autocompleteOnly": [4, "autocomplete-only"],
    "triggerEl": [16],
    "offset": [16],
    "placement": [1],
    "isOpen": [1540, "is-open"],
    "squared": [4],
    "openMenu": [64],
    "closeMenu": [64]
  }, [[0, "mxClick", "onMenuItemClick"], [6, "click", "onClick"], [6, "focus", "onFocus"], [4, "keydown", "onDocumentKeyDown"], [0, "keydown", "onKeydown"]]]);
function defineCustomElement() {
  if (typeof customElements === "undefined") {
    return;
  }
  const components = ["mx-menu"];
  components.forEach(tagName => { switch (tagName) {
    case "mx-menu":
      if (!customElements.get(tagName)) {
        customElements.define(tagName, MxMenu);
      }
      break;
  } });
}

export { MxMenu as M, defineCustomElement as d };

//# sourceMappingURL=mx-menu2.js.map